#!/bin/sh

# Always call wrapper mode under R64!
DIST_OS=`uname -s | tr [:upper:] [:lower:] | tr -d [:blank:]`
# To run nonwrapper mode under AIX, change below line to:   if [ $DIST_OS = "" ]
if [ $DIST_OS = "aix" ]
then
  R64_FINDNAME=$0 
  while [ -h $R64_FINDNAME ] ; do R64_FINDNAME=`ls -ld $R64_FINDNAME | awk '{print $NF}'` ; done 
  R64_RUNDIR=`echo $R64_FINDNAME | sed -e 's@/[^/]*$@@'` 
  unset R64_FINDNAME
  HQ_JAVA_HOME=C:\Program Files\SASHome\SASPrivateJavaRuntimeEnvironment\9.4\jre
  export HQ_JAVA_HOME
  if test -d $R64_RUNDIR; then
    $R64_RUNDIR/hq-agent.sh "$@"
  else
    ./hq-agent.sh "$@"
  fi
  exit 0
fi

AGENT_BUNDLE_HOME_PROP=agent.bundle.home
AGENTPROPFILE_PROP=agent.propFile
AGENT_PROPS=../../conf/agent.properties
AGENTLOGDIR_PROP=agent.logDir
AGENTLOGDIR=../../log
AGENTDATADIR_PROP=agent.dataDir
AGENTDATADIR=../../data
AGENT_LIB=./lib
PDK_LIB=./pdk/lib
WRAPPER_LIB=../../wrapper/lib
WRAPPER_DIR=../../wrapper
# for /proc/net/tcp mirror
SIGAR_PROC_NET=./tmp

# ------------- 
# Shouldn't need to change anything below this
# -------------

FINDNAME=$0 
while [ -h $FINDNAME ] ; do FINDNAME=`ls -ld $FINDNAME | awk '{print $NF}'` ; done 
RUNDIR=`echo $FINDNAME | sed -e 's@/[^/]*$@@'` 
unset FINDNAME
if test -d $RUNDIR; then
  cd $RUNDIR/..
else
  cd ..
fi

HQ_JAVA_HOME=C:\Program Files\SASHome\SASPrivateJavaRuntimeEnvironment\9.4\jre
export HQ_JAVA_HOME

if [ "x${HQ_JAVA_HOME}" != "x" ] ; then
    HQ_JAVA_HOME=${HQ_JAVA_HOME}
elif [ -d "${AGENT_BUNDLE_HOME}"/jre ]; then
    HQ_JAVA_HOME="${AGENT_BUNDLE_HOME}"/jre
    # Just in case
    chmod -R +x "${AGENT_BUNDLE_HOME}"/jre/bin/* > /dev/null 2>&1
elif [ -d jre ]; then
    HQ_JAVA_HOME=jre
    # Just in case
    chmod -R +x jre/bin/* > /dev/null 2>&1
elif [ "x$JAVA_HOME" != "x" ] ; then
    HQ_JAVA_HOME=${JAVA_HOME}
else
    case "`uname`" in
    Darwin)
        HQ_JAVA_HOME=/System/Library/Frameworks/JavaVM.framework/Home
        ;;
    *)
        echo "HQ_JAVA_HOME or JAVA_HOME must be set when invoking the agent"
        exit 1
        ;;
    esac
fi

chmod +x ./pdk/scripts/* > /dev/null 2>&1

HQ_JAVA="${HQ_JAVA_HOME}/bin/java"

CLIENT_CLASSPATH=
for i in `ls ${AGENT_LIB}/*.jar`
do
  CLIENT_CLASSPATH="${CLIENT_CLASSPATH}:${i}"
done

for i in `ls ${PDK_LIB}/*.jar`
do
  CLIENT_CLASSPATH="${CLIENT_CLASSPATH}:${i}"
done

for i in `ls ${WRAPPER_LIB}/*.jar`
do
  CLIENT_CLASSPATH="${CLIENT_CLASSPATH}:${i}"
done

AGENT_INSTALL_HOME=C:\SAS\Config\Lev1\Web\SASEnvironmentManager/agent-5.8.0-EE

CLIENT_CLASSPATH="${CLIENT_CLASSPATH}:${AGENT_LIB}"

CLIENT_CLASS=org.hyperic.hq.bizapp.agent.client.AgentClient

HQ_JAVA_OPTS="${HQ_JAVA_OPTS} \
    -Xmx128m \
    -Dsun.net.inetaddr.ttl=60 \
    -Djava.net.preferIPv4Stack=true \
    -Djava.security.auth.login.config=jaas.config \
    -Dagent.install.home=${AGENT_INSTALL_HOME} \
    -Dagent.bundle.home=${AGENT_INSTALL_HOME}/bundles/agent-5.8.0 \
    -Dhttps.protocols=TLSv1.2 \
    -Djdk.tls.client.protocols=TLSv1.2 \
    -Djava.library.path=${WRAPPER_LIB} \
    -D${AGENTPROPFILE_PROP}=${AGENT_PROPS} \
    -D${AGENTLOGDIR_PROP}=${AGENTLOGDIR} \
    -D${AGENTDATADIR_PROP}=${AGENTDATADIR}"

CLIENT_CMD="${HQ_JAVA} \
    ${HQ_JAVA_OPTS} \
    -cp ${CLIENT_CLASSPATH} ${CLIENT_CLASS}"

START_CMD="${CLIENT_CMD} start"
STATUS_CMD="${CLIENT_CMD} status"
PING_CMD="${CLIENT_CMD} ping"
SETUP_CMD="${CLIENT_CMD} setup"
DIE_CMD="${CLIENT_CMD} die"
SET_PROP_CMD="${CLIENT_CMD} $@"

# Resolve the location of the 'ps' command
PSEXE="/usr/bin/ps"
if [ ! -x "$PSEXE" ]
   then
   PSEXE="/bin/ps"
   if [ ! -x "$PSEXE" ]
      then
      echo "Unable to locate 'ps'."
      echo "Please report this message along with the location of the command on your system."
      exit 1
   fi
fi
    
PIDFILE="${WRAPPER_DIR}/hq-agent.pid"
PID=""
#PID=`$PSEXE -ef -o pid -o args | grep "hq-agent-core-5.8.0" | grep -v grep | tail -1 | awk '{print $1}'`

getpid() {
    if [ -f "$PIDFILE" ]
    then
        if [ -r "$PIDFILE" ]
        then
            PID=`cat "$PIDFILE"`
            if [ "X$PID" != "X" ]
            then
                # This file is only working for AIX R64 platform.
                pidtest=`$PSEXE -p $PID -o args | grep "hq-agent-core-5.8.0" | tail -1`
                if [ "X${pidtest}" = "X" ]
                then
                    # This is a stale pid file.
                    rm -f "$PIDFILE"
                    echo "Removed stale pid file: $PIDFILE"
                    PID=""
                fi
            else
                # This is a empty pid file.
                rm -f "$PIDFILE"
                echo "Removed empty pid file: $PIDFILE"
                PID=""
            fi
        else
            echo "Cannot read $PIDFILE."
            exit 1
        fi
    fi
}

if [ "$1" = "start" ] ; then
    echo "Starting agent"
    getpid
    if [ "X$PID" = "X" ]
    then
        opt=
        #opt=" >/dev/null 2>&1"
        ncmd=${START_CMD}$opt
        eval $ncmd &
    else 
        echo "Agent is already running."
        exit 1
    fi
    
    # Sleep for a few seconds to allow for intialization if required 
    #  then test to make sure we're still running.
    #
    i=0
    started=0
    WAIT_AFTER_STARTUP=30
    while [ $i -lt $WAIT_AFTER_STARTUP ]
    do
        sleep 1
        i=`expr $i + 1`
        PID=`$PSEXE -ef -o pid -o args | grep "${AGENT_INSTALL_HOME}" | grep -v grep | tail -1 | awk '{print $1}'`
        if [ "X$PID" = "X" ]
        then
            echo ".\c"
        else
            i=$WAIT_AFTER_STARTUP
            started=1
        fi
    done
    
    echo "."

    if [ started -eq 0 ]
    then
        echo " WARNING: Agent may have failed to start. Timed out."
        exit 1
    else
        # Create PID file.
        echo $PID > $PIDFILE
        echo " running ($PID)."
        exit 0
    fi
    
elif [ "$1" = "run" ] ; then
    echo "Running agent"
    echo ${START_CMD} 
    ${START_CMD} 
elif [ "$1" = "stop" ] ; then
    rm -f "$PIDFILE"
    echo "Removed pid file: $PIDFILE"
    ${DIE_CMD} 10 || exit 1
elif [ "$1" = "status" ] ; then
    getpid
    ${STATUS_CMD} || exit 1
elif [ "$1" = "ping" ] ; then
    echo Pinging ...
    VAL=`$PING_CMD 2>&1`
    if [ "$?" = "0" ] ; then
        echo 'Success!'
    else
        echo 'Failure!'
        echo $VAL
        exit 1
    fi
elif [ "$1" = "setup" ] ; then
    ${SETUP_CMD}
elif [ "$1" = "set-property" ] ; then
    ${SET_PROP_CMD}
else
    echo "Syntax: $0 "'<start | stop | ping | setup | set-property>'
    exit 1
fi

# PID=`$PSEXE -ef -o pid -o args | grep "hq-agent-core-5.8.0" | grep -v grep | tail -1 | awk '{print $1}'`
#
# if [ "X$PID" = "X" ]
# then
#   # Process is gone so remove the pid file.
#   rm -f "$PIDFILE"
#   PID=""
# fi
