#!/usr/bin/perl

# Perl script to create and EMIping service to monitor successful ICMP ping
# from a client with a running agent to a user selected hostname or web service assuming
# a form of the ping command was used

use warnings;
use strict;
use Getopt::Long;
use Pod::Usage;

# Constants
use constant {
	AUTHOR  => 'bnb',
	VERSION => '$Id: ping 2013-04-04 21:00:07Z bnb $ ',
	LOG_TAG => '',
};

our %globalsavespace;
my ( $pingcmd, $pingtarget, $pingcount );

( $pingcmd, $pingtarget, $pingcount ) = get_command_line_arguments( \%globalsavespace );
$| = 1;

my $pingresult = `$pingcmd $pingtarget $pingcount`;
if ( $? != 0 ) {
	pod2usage("$0 pingcmd execution error, rc=$?");
	exit 2;
}
exit 0;

sub get_command_line_arguments {
	my ($globalsavespace_ref) = @_;
	my ( $pingtarget, $pingcmd, $opt_c, $opt_m, $opt_h );

	if ( @ARGV > 0 ) {
		GetOptions(
			'count|c:s' => \$opt_c,
			'help|h!'    => \$opt_h,
			'man|m!' => \$opt_m,
		  )
		  or pod2usage(-msg => "Try '$0 --help' for more information.", -exitval => 2);
	}

    pod2usage(-verbose => 0, -exitval => 0) if $opt_h;
    pod2usage(-verbose => 1, -exitval => 0) if $opt_m;

	if ($opt_c) {
	    $opt_c= '-c ' . $opt_c;
	}
	else {
		$opt_c= '-c 3';
	}

	if ( scalar (@ARGV) == 2) {
		( $pingcmd, $pingtarget ) = @ARGV;
	}
	else { pod2usage(-verbose => 0, -exitval => 2);
	}

	return ( $pingcmd, $pingtarget, $opt_c );
}   # end of get_command_line_arguments

=head1 NAME

 ping.pl

=head1 SYNOPSIS

 ping.pl [-c count ] [ -h ] pingcmd target

=head1 DESCRIPTION

 SAS EV echo ping function to confirm host or website is reachable from platform.

 Switches that don't define a value can be done in long or short form.
 eg:
   ping.pl --help
   ping.pl -h

=head1 ARGUMENTS

 ping command
 
 --help      print Options and Arguments
 --man       print complete man page

 Target can be individual hostname:
   foo or FQDN name foo.sas.com or website www.sas.com

=head1 OPTIONS

 --count      number of ping (ICMP) packets
 --versions   print Modules, Perl, OS, Program info
 --debug 0    don't print debugging information (default)
 --debug 1    print debugging information

=head1 AUTHOR

bnb