#!/bin/bash

# Copyright (c) 2013 SAS Institute Inc. 
# Unpublished - All Rights Reserved 
#
# pgha.sh
# This script verifies and/or reestablishes postgres master slave HA 

#
# USAGE
# Usage: pgha.sh [-d datapath ] [ -h ] [ -l logname ] [ -p path ] [ -u user ] [ -t triggerfile ] status | promote    
#
# RETURN CODES
# 0   : Success
# 2   : Failure  -  Needed for agent red failure condition

# ====================================================================
# Revision history
# ====================================================================
# Date        Developer   Change
# ----        ---------   ------
# 2013-03-18  bnb      	  Coded
# 2013-01-05  bnb		  Finished trigger testing, updates error cleanups

SCRIPT_NAME="SAS Postgres HA script"
SCRIPT_VERSION="9.4.0.20130322"
SCRIPT_BUILD_ID="201303221500"

SCRIPT_PATH=$(dirname $(readlink -f $0))
_LOG_FILENAME="$(basename $(readlink -f $0) .sh)_scripts.log"

# Include common functions
. "${SCRIPT_PATH}/emi_include.sh"

LogDebug "Log Directory    = ${_LOG_DIR}"
LogDebug "Log Filename     = ${_LOG_FILENAME}"

#Log "== START =="
#LogTerm "Script Location = ${_EMI_COMMON_PATH}"
#LogTerm "Agent Directory = ${_AGENT_DIRECTORY}"
#LogTerm "Plugin Name     = ${_PLUGIN_NAME}"
#LogTerm "Log Directory   = ${_LOG_DIR}"
#LogTerm "Log Filename    = ${_LOG_FILENAME}"
#LogTrace "Test TRACE message 1"
#LogDebug "Test LogDebug message 1"
#LogInfo "Test INFO message 1"
#LogWarn "Test WARN message 1"
#LogError "Test ERROR message 1"
#Log "==  END  =="

# ====================================================================
# FUNCTIONS
# ====================================================================
ShowUsage() {
    LogTerm "Usage: $SCRIPT_FILENAME [-d datapath ] [ -h ] [ -l logname ] [ -i path ] [ -p port ] [ -u user ] [ -t triggerfile ] status | promote"
    exit 2
}

PG_USER="postgres"
PG_PORT="5432"
# ====================================================================
# Pulling off options, prepping for execution of requested function 
# ====================================================================
while : 
do
	LogDebug "Looping $1"
	case "$1" in
		#-d | --data)
			#PG_DATADIR="$2"
			#shift 2
			#if ! [ -d "$PG_DATADIR" ]; then
				#	LogTermError "DATA directory missing [$PG_DATADIR]"
				#ShowUsage
			#fi
			#;;
		-h | --help)
			ShowUsage
			exit 2
			;;
		-i | --install)
			PG_PATH="$2"
			shift 2
			if ! [ -f "$PG_PATH/bin/psql" ]; then
				LogTermError "Cannot find psql in install bin [$PG_PATH]"
				ShowUsage
			fi
			;;
		-l | --log)
			if [ "$2" == "default" ]; then
				shift 2
			else
				_LOG_FILENAME="$(basename $(readlink -f $2))"
				_LOG_DIR="$(dirname $(readlink -f $2))"
				shift 2
			fi
			LogDebug "LOGBASE=$_LOG_FILENAME LOGDIR [$_LOG_DIR]"
			;;
		-p | --port)
			PG_PORT="$2"
			LogDebug "Postgres TCP/IP Communications port=$PG_PORT"
			shift 2
			;;
		status)
			if [ -n "$PG_CHOICE" ]; then
				LogTermError "Bad syntax $PG_CHOICE invalid"
				ShowUsage
			fi
			PG_CHOICE="status"
			shift
			;;
		promote)
			if [ -n "$PG_CHOICE" ]; then
				LogTermError "Bad syntax $PG_CHOICE invalid"
				ShowUsage
			fi
			PG_CHOICE="promote"
			shift
			;;
		-t | --trigger)
			PG_TRIGGERFILE="$2"
			LogDebug "Postgres Trigger File [$PG_TRIGGERFILE]"
			shift 2
			;;
		-u | --user)
			PG_USER="$2"
			shift 2
			;;
		-v | --verbose)
			verbose="verbose"
			_LOG_LEVEL=DEBUG
			shift
			;;
		--) # end of all options
			shift
			break;
			;;
		-*)	
			LogTerm "Error: Unknown option: $1"
			exit 2
			;;
		*)  # No more options
			break
			;;
	esac
done

if [ -z "$PG_CHOICE" ]; then
	LogTermError "Missing mandatory argument status | promote"
	ShowUsage
fi

LogDebug "Using user $PG_USER $PG_PORT $PG_PATH"
if [ -z "$PG_PATH" ]; then
	PG_PATH="/usr"
	# PG_PATH="/var/lib/postgres"
	LogDebug "Using default postgres install path [/usr]"
	if ! [ -f "$PG_PATH/bin/psql" ]; then
		LogTermError "Cannot find psql in [$PG_PATH]"
		ShowUsage
	fi
fi
			
#if [ -z "$PG_DATADIR" ]; then
	#PG_DATADIR="/var/lib/pgsql/data"
	#LogDebug "Using default postgres data path [/var/lib/pgsql]"
	#if ! [ -d "$PG_DATADIR" ]; then
		#	LogTermError "DATA directory missing at [$PG_DATADIR]"
		#	ShowUsage
	#fi
#fi

# ======================================================
# need to remember original and clean up after myself

export LD_LIBRARY_PATH=$PG_PATH/lib:$LD_LIBRARY_PATH
# ======================================================

#
# Test to see if postgres is running by a query to pg_stat_activity
# Also testing to see if its a master or slave.
#
curr_conn=`$PG_PATH/bin/psql -U $PG_USER -p $PG_PORT -qAtX --command="SELECT COUNT(*) from pg_stat_activity;" | \
sed s/\|/\=/`
rc=$?
if [[ -z $curr_conn ]]; then
	LogTermError "Database exist test failed, curr_conn=$curr_conn"
	exit 2
fi
if [ "$rc" != "0" ]; then
	LogTermError "Bad sed return code, curr_conn=$curr_conn rc=$rc"
	exit 2
fi

# max connection query
max_conn=`$PG_PATH/bin/psql -U $PG_USER -p $PG_PORT -qAtX --command="SHOW max_connections;"`
rc=$?
if [ "$rc" != "0" ]; then
	LogTermError "Database max_conn query failed max_conn=$max_cunn, Bad return code=$rc"
	exit 2
fi
LogDebug "$PG_USER pg_stat_activity curr_connection=$curr_conn max_connection=$max_conn"

pctused=`echo "scale=2; $curr_conn / $max_conn" | bc `
#
# So database exists, now test for slave LSN value on master via pg_current_xlog_location query 
# master or pg_last_xlog_receive_location on slave to determine if I am on master or slave
#
PG_TYPE="Unknown"
slavechk=""
slavechk=`$PG_PATH/bin/psql -U $PG_USER -p $PG_PORT -qAtX -c "SELECT pg_is_in_recovery();" 2>/dev/null`
if [ "f" == "$slavechk" ]; then
	PG_TYPE="master"
	slavechk=`$PG_PATH/bin/psql -U $PG_USER -p $PG_PORT -qAtX -c "SELECT pg_current_xlog_location();"| \
	awk -F/ 'BEGIN{print "ibase=16"} {printf "%s%08s\n", $1, $2}' | bc `
	LogInfo "Master pg_current_xlog_location=$slavechk pg_stat_activity connections=$curr_conn max=$max_conn"
else if [ "t" == "$slavechk" ]; then
		PG_TYPE="slave"
		slavechk=`$PG_PATH/bin/psql -U $PG_USER -p $PG_PORT -qAtX -c "SELECT pg_last_xlog_receive_location();"| \
		awk -F/ 'BEGIN{print "ibase=16"} {printf "%s%08s\n", $1, $2}' | bc `
		LogInfo "Slave pg_xlog_receive_location=$slavechk pg_stat_activity connections=$curr_conn max=$max_conn"
	else 
		LogTermError "pg_current_xlog & pg_last_xlog_receive tests failed, Return code=$rc value [$slavechk]"
		exit 2
	fi
fi

case $PG_CHOICE in
	"status")
		LogDebug "Results output pgcurrentxlog=$slavechk current=$curr_conn pctused=$pctused"
		if [ "$PG_TYPE" == "master" ]; then
			printf '%s\n%s\n%s\n' "pgcurrentxlog=$slavechk" "current=$curr_conn" "pctused=$pctused"
		else
			printf '%s\n%s\n%s\n' "pglastxlog=$slavechk" "current=$curr_conn" "pctused=$pctused"
		fi
		exit 0
       ;;
    "start")
       ;;
    "stop")
    	;;
    "promote")
    	LogTermInfo "Promoting to master, trigger file [$PG_TRIGGERFILE];"
		if [ "$PG_TYPE" == "master" ]; then
			LogTermError "On $PG_TYPE, Cannot promote master to master"
			exit 2	
		fi
    	if [ "$PG_TYPE" == "slave" ]; then
			if [ -f $PG_TRIGGERFILE ]; then
    			LogTermError "Triggerfile [$PG_TRIGGERFILE] exists before creation!"
				exit 2
			else # creating trigger, checking success
				touch $PG_TRIGGERFILE
    			if ! [ -f $PG_TRIGGERFILE ]; then
    				LogTermError "Could not create [$PG_TRIGGERFILE]"
    				exit 2
    			fi
    				sleep 5
    			LogDebug "Created triggerfile [$PG_TRIGGERFILE]"
			fi
			# now should confirm status t
			slavechk=`$PG_PATH/bin/psql -U $PG_USER -p $PG_PORT -qAtX -c "SELECT pg_is_in_recovery();" 2>/dev/null`
			rc=$?
			if [ "f" == "$slavechk" ]; then
				LogTermInfo "Master confirmed, Promote successful"
				exit 0
			else
				LogTermError "Promote failed, psql pg_is_in_recovery() returns=[$slavechk]"
				exit 2
			fi
		fi
    	;;
    *)  # should never get here, but included for completeness
        LogTermError "ERROR: Invalid internal argument, should never get there PG_CHOICE=$PG_CHOICE"
        ;;
esac
LogDebug "=========== END OF EXECUTION =============="
