// SAS Report Center Controller

import org.hyperic.hq.context.Bootstrap
import org.hyperic.hq.hqu.rendit.BaseController
import org.hyperic.hq.appdef.shared.PlatformManager
import org.hyperic.hq.appdef.shared.ServerManager
import org.hyperic.hq.measurement.shared.MeasurementManager
import org.hyperic.util.pager.PageControl
import org.hyperic.hq.authz.shared.PermissionException

import org.apache.commons.logging.Log
import org.apache.commons.logging.LogFactory

class ReportcenterController extends BaseController
{
    Log log = LogFactory.getLog(this.class)

    private pMan = Bootstrap.getBean(PlatformManager.class)
    private sMan = Bootstrap.getBean(ServerManager.class)
    private mMan = Bootstrap.getBean(MeasurementManager.class)

    // Called when someone navigates to Report Center
    def index(params) {
        InetAddress addr = InetAddress.getLocalHost()
        def hostname = addr.getCanonicalHostName()
        def emiEnabled = false
        def stpURL = ""

        log.debug "Report Center index()"
        log.debug "    hostname = $hostname"

        // Determine if Service Architecture is initialized
        def platform = pMan.getPlatformByName(hostname)
        log.debug "platform = $platform"

        def servers = []
        def matching = resourceHelper.find('byPrototype': 'SAS Environment Manager Data Mart 9.4')
        for (resource in matching) {
            log.debug "Found resource $resource (${resource.class.name})"
            try {
                servers.add(resource)
            } catch (PermissionException e) {
                log.debug("Ignoring resource " + resource.name + " due to permissions")
            }
        }

        if (servers.size() > 0) {
            def dmSvr = servers[0]
            log.debug "Checking resource: $dmSvr"
            emiEnabled = isInitialized(dmSvr.toServer())
            stpURL = dmSvr.toServer().resource.config['stpURL']?.value ?: ""
        }

        log.info "Launching Report Center view with URL: $stpURL, and emiEnabled=$emiEnabled"
        render(locals: [stpURL:stpURL, emiEnabled:emiEnabled])
    }

    def isInitialized(svr) {
        if (log.debugEnabled) {
            log.debug "svr        = $svr"
            log.debug "svr config = $svr.resource.config"
            log.debug "svr init   = ${svr.resource.config['initialized']?.value}"
        }

        svr.resource.config['initialized']?.value =~ /Yes/
    }
}
