/**
 *  SAS
 *  Environment Snapshot Controller
**/

import org.hyperic.hq.appdef.server.session.AgentSortField
import org.hyperic.hq.appdef.shared.AppdefEntityConstants
import org.hyperic.hq.appdef.shared.AppdefEntityValue
import org.hyperic.hq.appdef.shared.AppdefEntityID
import org.hyperic.hq.authz.shared.AuthzSubjectManager
import org.hyperic.hq.hqu.rendit.BaseController
import org.hyperic.hq.context.Bootstrap
import org.hyperic.util.config.ConfigResponse
import org.hyperic.hq.appdef.shared.ConfigManager
import org.hyperic.hq.appdef.shared.CPropManager
import org.hyperic.hq.appdef.server.session.ConfigResponseDAO
import org.hyperic.hq.control.server.session.ControlHistoryDAO
import org.hyperic.hq.control.server.session.ControlScheduleDAO
import java.text.DateFormat
import org.hyperic.hq.hqu.rendit.html.DojoUtil
import org.hyperic.hq.common.Humidor
import org.hyperic.hq.events.shared.EventLogManager
import org.hyperic.hq.events.server.session.EventLogSortField
import org.hyperic.hq.events.EventLogStatus
import org.hyperic.hq.hqu.rendit.util.HQUtil
import javax.servlet.http.HttpServletRequest
import org.hyperic.hq.measurement.server.session.MeasurementDAO
import org.hyperic.hq.measurement.shared.MeasurementManager
import org.hyperic.util.pager.PageControl
import org.hyperic.hibernate.PageInfo
import org.hyperic.hq.appdef.shared.PlatformManager
import org.hyperic.hq.product.shared.PluginManager
import org.hyperic.util.PrintfFormat
import org.hyperic.hq.bizapp.shared.ProductBoss
import org.hyperic.hq.authz.server.session.ResourceDAO
import org.hyperic.hq.authz.shared.ResourceGroupManager
import org.hyperic.hq.hqu.rendit.helpers.ResourceHelper
import org.hyperic.hq.authz.shared.ResourceManager
import org.hyperic.hq.authz.server.session.ResourceSortField
import org.hyperic.hq.common.shared.ServerConfigManager
import org.hyperic.hq.appdef.server.session.ServerDAO
import org.hyperic.hq.appdef.server.session.ServerTypeDAO
import org.hyperic.hq.appdef.shared.ServerManager
import org.hyperic.hq.appdef.shared.ServiceManager
import org.hyperic.hq.appdef.server.session.ServiceTypeDAO
import org.hyperic.hq.appdef.server.session.ServiceDAO
import java.text.SimpleDateFormat
import java.lang.String
import org.hyperic.util.units.UnitsConstants
import org.hyperic.util.units.UnitsFormat
import org.hyperic.util.units.UnitNumber
import groovy.xml.*
import groovy.time.*

class SnapshotController extends BaseController {
	
	// Formatting variables used for printing
	// data into snapshot reports
	private final DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT)
	private final PrintfFormat ctrlHisDataFmt = new PrintfFormat("%-110s %-25s %-15s %-20s %-15s %-55s")
	private final PrintfFormat ctrlSchedDataFmt = new PrintfFormat("%-110s %-25s %-15s %-20s")
	private final PrintfFormat dataFmt = new PrintfFormat("%-65s %-105s")
	private final PrintfFormat dfDataFmt = new PrintfFormat("%-50s %-18s %-18s %-18s %-18s %-40s %-30s")
	private final PrintfFormat eventDataFmt = new PrintfFormat("%-30s %-20s %-80s %-47s %-65s")
	private final PrintfFormat fileMountDataFmt = new PrintfFormat("%-105s %-30s %-30s %-30s %-30s")
	private final PrintfFormat netIntDataFmt = new PrintfFormat("%-65s %-31s %-31s %-31s %-31s")
	private final PrintfFormat nfsMountDataFmt = new PrintfFormat("%-105s %-72s")
	private final PrintfFormat topDataFmt = new PrintfFormat("%-75s %-17s %-24s %-17s %-17s %-17s %-17s %-17s %-17s %-17s")
	private final PrintfFormat whoDataFmt = new PrintfFormat("%-30s %-40s %-40s %-30s")
	
	// Bootstraps for java and groovy classes
	//private appdefMan = Bootstrap.getBean(AppdefEntityID.class)
	private configMan = Bootstrap.getBean(ConfigManager.class)
	private configDaoMan = Bootstrap.getBean(ConfigResponseDAO.class)
	private cpropMan = Bootstrap.getBean(CPropManager.class)
	private ctrlHisDaoMan = Bootstrap.getBean(ControlHistoryDAO.class)
	private ctrlSchedDaoMan = Bootstrap.getBean(ControlScheduleDAO.class)
	private eMan = Bootstrap.getBean(EventLogManager.class)
	private gMan = Bootstrap.getBean(ResourceGroupManager.class)
	private mMan = Bootstrap.getBean(MeasurementManager.class)
	private mDaoMan = Bootstrap.getBean(MeasurementDAO.class)
	private platMan = Bootstrap.getBean(PlatformManager.class)
	private pluginMan = Bootstrap.getBean(PluginManager.class)
	private prodBossMan = Bootstrap.getBean(ProductBoss.class)
	private rDaoMan = Bootstrap.getBean(ResourceDAO.class)
	private rMan = Bootstrap.getBean(ResourceManager.class)
	private sConfMan = Bootstrap.getBean(ServerConfigManager.class)
	private sDaoMan = Bootstrap.getBean(ServerDAO.class)
	private sTypeDaoMan = Bootstrap.getBean(ServerTypeDAO.class)
	private sMan = Bootstrap.getBean(ServerManager.class)
	private svcTypeDaoMan = Bootstrap.getBean(ServiceTypeDAO.class)
	private svcDaoMan = Bootstrap.getBean(ServiceDAO.class)
	private svcMan = Bootstrap.getBean(ServiceManager.class)
	
	// Only allows super users to use plugin
	// and sets the JSON methods that are
	// called from this controller
	SnapshotController() {
		onlyAllowSuperUsers()
		setJSONMethods(['getPlat', 'hardwareData', 'cpuData', 'systemData', 'networkData', 'netintData', 'filemountData', 'nfsmountData',
						'serverData', 'serviceData', 'logData', 'ctrlhisData', 'ctrlschedData', 'connspawnData', 'metaData', 'olapData',
						'objspawnData', 'storedprocData', 'workspaceData', 'pooledworkspaceData', 'osservicesData', 'activemqData',
						'tomcatData', 'tcruntimeData', 'licenseData','topData', 'dfData', 'whoData', 'snapshotData', 'eventData', 
						'alertData', 'pluginData', 'datamartData', 'acmData', 'apmData'])
	}
	
	// Schemas for the tables that are displayed
	// in the index page -- format of schema can
	// be found in DojoUtil.groovy within the
	// source code
	private HARDWARE_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getHardwareData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private CPU_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getCpuData(platName)
		},
		rowId: {it.row},
		columns: [
			[field: [getValue: {localeBundle.processor},
			 description:'processor', sortable:false], 
			 width: '50%',
			 label: {it.resource}],
			 
			[field: [getValue: {localeBundle.cpuUsage},
			 description:'cpu usage', sortable:false], 
			 width: '50%',
			 label: {it.value}],
		]
    ]
	
	private SYSTEM_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getSystemData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
	]
	
	private NETWORK_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getNetworkData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property}, 
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
	]
	
	private NETINT_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getNetintData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.netInt},
			 width: '24%',
			 label: {it.resource}],
			 
			[field: [getValue: {localeBundle.brps},
			 description:'bits received per second', sortable:false], 
			 width: '19%',
			 label: {it.brps}],
			 
			[field: [getValue: {localeBundle.btps},
			 description:'bits transmitted per second', sortable:false], 
			 width: '19%',
			 label: {it.btps}],
			 
			[field: [getValue: {localeBundle.prpm},
			 description:'packets received per minute', sortable:false], 
			 width: '19%',
			 label: {it.prpm}],
			 
			[field: [getValue: {localeBundle.ptpm},
			 description:'packets transmitted per minute', sortable:false], 
			 width: '19%',
			 label: {it.ptpm}],
		]
    ]
	
	private FILEMOUNT_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getFilemountData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.mount},
			 width: '32%',
			 label: {it.resource}],
			 
			[field: [getValue: {localeBundle.drpm},
			 description:'disk reads per minute', sortable:false], 
			 width: '17%',
			 label: {it.drpm}],
			 
			[field: [getValue: {localeBundle.dwpm},
			 description:'disk writes per minute', sortable:false], 
			 width: '17%',
			 label: {it.dwpm}],
			 
			[field: [getValue: {localeBundle.tbf},
			 description:'total bytes free', sortable:false], 
			 width: '17%',
			 label: {it.tbf}],
			 
			[field: [getValue: {localeBundle.up},
			 description:'use percent', sortable:false], 
			 width: '17%',
			 label: {it.up}],
		]
    ]
	
	private NFSMOUNT_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getNfsmountData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.mount},
			 width: '49%',
			 label: {it.resource}],
			 
			[field: [getValue: {localeBundle.resTime},
			 description:'response time', sortable:false], 
			 width: '51%',
			 label: {it.value}],
		]
    ]
	
	private SERVER_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getServerData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME, 
			 width: '50%',
			 label: {it.server}],
			 
			[field: [getValue: {localeBundle.count},
			 description:'server count', sortable:false], 
			 width: '50%',
			 label: {it.count}],
		]
    ]
	
	private SERVICE_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getServiceData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME, 
			 width: '50%',
			 label: {it.service}],
			 
			[field: [getValue: {localeBundle.count},
			 description:'service count', sortable:false],
			 width: '50%',
			 label: {it.count}],
		]
    ]
	
	private LOG_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getLogData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.server},
			 width: '50%',
			 label: {it.server}],
			 
			[field: [getValue: {localeBundle.logLocation},
			 description:'location', sortable:false], 
			 width: '50%',
			 label: {it.location}],
		]
	]
	
	private CTRLHIS_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getCtrlhisData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.resource},
			 width: '25%',
			 label: {it.resource}],
			 
			[field: [getValue: {localeBundle.dateTime},
			 description:'date and time', sortable:false], 
			 width: '8%',
			 label: {it.dateTime}],
			 
			[field: [getValue: {localeBundle.status},
			 description:'status', sortable:false], 
			 width: '8%',
			 label: {it.status}],
			 
			[field: [getValue: {localeBundle.action},
			 description:'action', sortable:false], 
			 width: '8%',
			 label: {it.action}],
			 
			[field: [getValue: {localeBundle.args},
			 description:'args', sortable:false], 
			 width: '8%',
			 label: {it.args}],
			 
			[field: [getValue: {localeBundle.message},
			 description:'message', sortable:false], 
			 width: '43%',
			 label: {it.message}],
		]
    ]
	
	private CTRLSCHED_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getCtrlschedData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.resource},
			 width: '38%',
			 label: {it.resource}],
			 
			[field: [getValue: {localeBundle.dateTime},
			 description:'date and time', sortable:false], 
			 width: '8%',
			 label: {it.dateTime}],
			 
			[field: [getValue: {localeBundle.action},
			 description:'action', sortable:false], 
			 width: '8%',
			 label: {it.action}],
			 
			[field: [getValue: {localeBundle.scheduler},
			 description:'scheduler', sortable:false], 
			 width: '46%',
			 label: {it.scheduler}],
		]
    ]
	
	private CONNSPAWN_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getConnspawnData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private META_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getMetaData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private OLAP_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getOlapData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private OBJSPAWN_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getObjspawnData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private STOREDPROC_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getStoredprocData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private WORKSPACE_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getWorkspaceData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]

    private POOLEDWORKSPACE_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getPooledworkspaceData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]

    private OSSERVICES_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getOsservicesData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private ACTIVEMQ_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getActivemqData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private TOMCAT_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getTomcatData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property},
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private TCRUNTIME_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getTcruntimeData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property}, 
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.config}],
		]
    ]
	
	private LICENSE_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getLicenseData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.property}, 
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.value}],
		]
	]
	
	private TOP_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getTopData(platName)
		},
		defaultSort: null,
		defaultSortOrder: null,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.cmd},
			 width: '28%',
			 label: {it.cmd}],
			 
			[field: AgentSortField.PORT,
			 header: {localeBundle.pid}, 
			 width: '7%',
			 label: {it.pid}],
			 
			[field: [getValue: {localeBundle.owner},
			 description:'owner', sortable:false], 
			 width: '12%',
			 label: {it.user}],
			 
			[field: [getValue: {localeBundle.stime},
			 description:'stime', sortable:false], 
			 width: '7%',
			 label: {it.stime}],
			 
			[field: [getValue: {localeBundle.size},
			 description:'size', sortable:false], 
			 width: '8%',
			 label: {it.size}],
			 
			[field: AgentSortField.CTIME,
			 header: {localeBundle.rss},
			 width: '8%',
			 label: {it.rss}],
			 
			[field: [getValue: {localeBundle.share},
			 description:'share', sortable:false],
			 width: '7%',
			 label: {it.share}],
			 
			[field: [getValue: {localeBundle.state},
			 description:'state', sortable:false], 
			 width: '7%',
			 label: {it.state}],
			 
			[field: [getValue: {localeBundle.uptime},
			 description:'time', sortable:false],
			 width: '8%',
			 label: {it.time}],
			 
			[field: AgentSortField.ADDR,
			 header: {localeBundle.cpu},
			 width: '8%',
			 label: {it.cpu}],
		]
	]
	
	private DF_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getDfData(platName)
		},
		defaultSort: null,
		defaultSortOrder: null,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.filesystem},
			 width: '22%',
			 label: {it.name}],
			 
			[field: [getValue: {localeBundle.sizegb},
			 description:'size', sortable:false],  
			 width: '13%',
			 label: {it.size}],
			 
			[field: [getValue: {localeBundle.usedgb},
			 description:'used', sortable:false], 
			 width: '13%',
			 label: {it.used}],
			 
			[field: [getValue: {localeBundle.availgb},
			 description:'avail', sortable:false], 
			 width: '13%',
			 label: {it.avail}],
			 
			[field: [getValue: {localeBundle.usePerc},
			 description:'usePerc', sortable:false], 
			 width: '13%',
			 label: {it.usePerc}],
			 
			[field: [getValue: {localeBundle.mount},
			 description:'mount', sortable:false], 
			 width: '13%',
			 label: {it.mount}],
			 
			[field: [getValue: {localeBundle.type},
			 description:'type', sortable:false],
			 width: '13%',
			 label: {it.type}],
		]
	]
	
	private WHO_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getWhoData(platName)
		},
		defaultSort: null,
		defaultSortOrder: null,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.user},
			 width: '25%',
			 label: {it.user}],
			 
			[field: [getValue: {localeBundle.device},
			 description:'device', sortable:false], 
			 width: '25%',
			 label: {it.device}],
			 
			[field: [getValue: {localeBundle.time},
			 description:'time', sortable:false], 
			 width: '25%',
			 label: {it.time}],
			 
			[field: [getValue: {localeBundle.host},
			 description:'host', sortable:false], 
			 width: '25%',
			 label: {it.host}],
		]
	]
	
	private SNAPSHOT_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getSnapshotData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME, 
			 width: '34%',
			 label: {it.name}],
			 
			[field: [getValue: {localeBundle.sizemb},
			 description:'file size', sortable:false], 
			 width: '33%',
			 label: {it.size}],
			 
			[field: [getValue: {localeBundle.modified},
			 description:'last modified', sortable:false], 
			 width: '33%',
			 label: {it.modified}],
		]
    ]
	
	private EVENT_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getEventData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.dateTime},
			 width: '10%',
			 label: {it.date}],
			 
			[field: [getValue: {localeBundle.status},
			 description:'status', sortable:false], 
			 width: '6%',
			 label: {it.status}],
			 
			[field: [getValue: {localeBundle.resource},
			 description:'resource', sortable:false], 
			 width: '25%',
			 label: {it.resource}],
			 
			[field: [getValue: {localeBundle.subject},
			 description:'subject', sortable:false], 
			 width: '25%',
			 label: {it.subject}],
			 
			[field: [getValue: {localeBundle.detail},
			 description:'detail', sortable:false], 
			 width: '34%',
			 label: {it.detail}],
		]
    ]
	
	private ALERT_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getAlertData(platName)
		},
		defaultSort: ResourceSortField.NAME,
		defaultSortOrder: 1,
		rowId: {it.row},
		columns: [
			[field: ResourceSortField.NAME,
			 header: {localeBundle.dateTime},
			 width: '10%',
			 label: {it.date}],
			 
			[field: [getValue: {localeBundle.status},
			 description:'status', sortable:false], 
			 width: '6%',
			 label: {it.status}],
			 
			[field: [getValue: {localeBundle.resource},
			 description:'resource', sortable:false], 
			 width: '25%',
			 label: {it.resource}],
			 
			[field: [getValue: {localeBundle.subject},
			 description:'subject', sortable:false], 
			 width: '25%',
			 label: {it.subject}],
			 
			[field: [getValue: {localeBundle.detail},
			 description:'detail', sortable:false], 
			 width: '34%',
			 label: {it.detail}],
		]
    ]

    private DATAMART_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getDatamartData(platName)
		},
		rowId: {it.row},
		columns: [
			[field: [getValue: {localeBundle.property},
			 description:'cprop', sortable:false], 
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.value}],
		]
    ]

    private ACM_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getAcmData(platName)
		},
		rowId: {it.row},
		columns: [
			[field: [getValue: {localeBundle.property},
			 description:'cprop', sortable:false], 
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.value}],
		]
    ]

    private APM_SCHEMA = [
		getData: {pageInfo, params ->
			def platName = params.getOne('plat')
			getApmData(platName)
		},
		rowId: {it.row},
		columns: [
			[field: [getValue: {localeBundle.property},
			 description:'cprop', sortable:false], 
			 width: '50%',
			 label: {it.property}],
			 
			[field: [getValue: {localeBundle.value},
			 description:'value', sortable:false], 
			 width: '50%',
			 label: {it.value}],
		]
    ]

    private formatTime(b) {
    	if ( b == -1 ) {
    		return 'Unlimited'
    	}

    	UnitsFormat.format(new UnitNumber(b, UnitsConstants.UNIT_DURATION, UnitsConstants.SCALE_MILLI), Locale.getDefault(), null).toString()
    }
	
	// Formats metrics to be a more 'user-friendly'
	// byte format
	private formatBytes(b) {
		if ( b.equals('Unlimited') ) {
			return b
		}
		else if ( b == -1 ) {
            return 'Unlimited'
		}
		
        UnitsFormat.format(new UnitNumber(b, UnitsConstants.UNIT_BYTES, UnitsConstants.SCALE_NONE), locale, null).toString()
    }
    
	// Formats percentages
    private formatPercent(n, total) {
        if ( total == 0 ){
			return 0
		}
		
        return (int)(n * 100 / total)
    }
	
	// Gathers an inventory summary for the
	// given environment
	private getInventoryData(params) {
        def types = []
        resourceHelper.findAppdefPrototypes().each { p ->
            types.add([name:  p.name, 
                       total: getResourceTypeCount(p)] )
        }
        return types
    }
	
	// Finds the count of a specific resource
	// type
	private getResourceTypeCount(p) {
        return resourceHelper.findByPrototype([byPrototype: p.name]).size()
    }
	
	// Used to format the status of events
	// for printing to tables and snapshots
	def checkStatus(status) {
		switch ( status ) {
			case 'ERR':
				status = "Error"
				break
			case 'WRN':
				status = "Warning"
				break
			case 'INF':
				status = "Info"
				break
			case 'ALR':
				status = "Alert"
				break
			case 'DBG':
				status = "Debug"
				break
			default:
				status = "?"
				break						
		}
		
		status
	}
	
	// Finds the availability of a resource --
	// currently only used to find the platform
	// availability for the live data methods
	def getAvailability(platform) {
		def m = mDaoMan.findByAliasAndID('Availability', platform)
		def avail = false
		if ( m.lastDataPoint ) {
			if ( m.lastDataPoint.value == 1 ) {
			avail = true
			}
		}
		
		return avail
	}
	
	def findPlatform(platform) {
		def p = resourceHelper.find(byFqdn: platform)
		if ( p ) {
			return p
		}
		/* else {
			for ( InetAddress localAddr : InetAddress.getAllByName(platform) ) {
				p = resourceHelper.find(byFqdn: localAddr.getHostAddress())
			}
		} */
		p
	}

	// Finds all of the hardware metrics and
	// config options for a platform
	def getHardwareData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def format 		= ['MemFree', 'SwapUsed']
		def noformat	= ['LoadAverage1']
		def metricOptions = []
		metricOptions += format
		metricOptions += noformat
		def configOptions = ['cpuSpeed', 'ram']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				configOptions.each{ c ->
					def config = p.config.get(c)
					
					if ( config ) {
						res << [row:row, property:config.description, config:config.value]
						row +=1
					}
				}
				
				metricOptions.each{ o ->
					def m = mDaoMan.findByAliasAndID(o, p)
					if ( m && m.lastDataPoint ) {
						def metric = m.lastDataPoint
						
						if ( format.contains(o) ) {
							metric = formatBytes(m.lastDataPoint.value)
						}
						
						res << [row:row, property:m.template.name, config:metric]
						row +=1
					}
				}
				
				if ( res.property ) {
					res.sort { a,b ->
						a.property <=> b.property
					}
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	// Finds all of the CPU metrics for a platform
	// -- queries the CPU platform services
	def getCpuData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		PageControl pc = PageControl.PAGE_ALL
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				// get the type information for CPU platform services
				def type = svcMan.findServiceTypeByName('CPU')
				if ( type ) {
					// once we have the 'CPU' type, get the platform
					// services of that type that exist
					def services = svcMan.getServicesByPlatform(user, p.entityId.id, type.id, pc)
					for ( s in services ) {
						def name = s.name
						if ( s.name.contains(p.name) ) {
							name = s.name.minus(p.name)
						}
						def metric = "?"
						
						// the simple 'findByAliasAndID' method that we use to
						// get metrics for platforms doesn't work for servers
						// or services, so we're stuck querying the metrics
						// this way
						def m = mMan.findEnabledMeasurements(user, s.entityId, '').find{ e ->
							e.template.name == 'Cpu Usage'
						}
						if ( m && m.lastDataPoint ) {
							metric = "${(m.lastDataPoint.value * 100).round(2)}%"
						}
						
						res << [row:row, resource:name, value:metric]
						row +=1
					}
				}
				try {
					if ( res.resource ) {
						res.sort { a,b ->
							// we want to sort by the number of the cpu
							// so we have to tokenize that value
							a.resource.tokenize()[2].toInteger() <=> b.resource.tokenize()[2].toInteger()
						}
					}
				}
				catch(e) {
					if ( res.resource ) {
						res.sort { a,b ->
							a.resource <=> b.resource
						}
					}
				}
			}
			else {
				res << [row:row, resource:'unavailable', value:'unavailable']
			}
		}
		res
	}
	
	// Finds all of the system metrics and config
	// options for a platform -- looks for 'System
	// Info' resource and queries that for ulimit
	def getSystemData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		PageControl pc = PageControl.PAGE_ALL
		def configOptions = ['ulimit.fileSize', 'ulimit.openFiles', 'ulimit.dataSegSize', 'ulimit.memorySize',  'ulimit.pipeSize',
							'ulimit.stackSize', 'ulimit.virtualMemory', 'vendor', 'vendorVersion']
		def metricOptions = ['processCount', 'threadCount']
		def pInfo = PageInfo.create(0, 9999, ResourceSortField.NAME, false)
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				// get all of the servers for a platform and
				// find all, if any, servers of type System Info
				def servers = p.getViewableChildren(user).find{ serv ->
					serv.prototype.name == 'SAS System Info'
				}
				
				configOptions.each{ c ->
					// check if the config option exists for the platform
					def config = p.config.get(c)
					if ( config ) {
						res << [row:row, property:config.description, config:config.value]
						row +=1
					}
					else {
						// if the config option doesn't exist for the
						// platform, check if the metric exists for the
						// servers gathered above
						for ( s in servers ) {
							def configs = s.config.get(c)
							if ( configs ) {
								def value = configs.value
								if ( value.toString().equals('-0') ) {
									value = 'Unlimited'
								}
								res << [row:row, property:configs.description, config:value]
								row +=1
							}
						}
					}
				}
				
				metricOptions.each{ o ->
					for ( s in servers ) {
						def m = mDaoMan.findByAliasAndID(o, s)
						if ( m && m.lastDataPoint ) {
							res << [row:row, property:m.template.name, config:m.lastDataPoint]
							row +=1
						}
					}
				}
				
				if ( res.property ) {
					res.sort { a,b ->
						a.property <=> b.property
					}
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	// Finds all of the network config options
	// for a platform
	def getNetworkData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['ip', 'primaryDNS', 'secondaryDNS', 'defaultGateway']
		
		if ( platform ) {
			def p = resourceHelper.find(byFqdn: platform)
			if ( p ) {
				configOptions.each{ o ->
					def config = p.config.get(o)
					
					if ( config ) {
						res << [row:row, property:config.description, config:config.value]
						row +=1
					}
				}
				
				if ( res.property ) {
					res.sort { a,b ->
						a.property <=> b.property
					}
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	// Finds all of the network interface metrics
	// for a platform -- queries the network interface
	// platform resources
	def getNetintData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		PageControl pc = PageControl.PAGE_ALL
		def brps = 'Bits Received per Second'
		def btps = 'Bits Transmitted per Second'
		def prpm = 'Packets Received per Minute'
		def ptpm = 'Packets Transmitted per Minute'
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				// get the type information for Network Server
				// Interface platform services
				def type = svcMan.findServiceTypeByName('NetworkServer Interface')
				if ( type ) {
					// once we have the Network Server Interface type,
					// get the platform services that exist of that type
					def services = svcMan.getServicesByPlatform(user, p.entityId.id, type.id, pc)
					for ( s in services ) {
						def name = s.name
						if ( s.name.contains(p.name) ) {
							name = s.name.minus(p.name)
						}
						
						def brpsVal = "?"
						def btpsVal = "?"
						def prpmVal = "?"
						def ptpmVal = "?"
						
						// the simple 'findByAliasAndID' method that we use to
						// get metrics for platforms doesn't work for servers
						// or services, so we're stuck querying the metrics
						// this way
						def metrics = mMan.findEnabledMeasurements(user, s.entityId, '')
						for ( m in metrics ) {
							if ( m.template.name.equals(brps) ) {
								if ( m.lastDataPoint ) {
									brpsVal = formatBytes(m.lastDataPoint.value)
								}
							}
							if ( m.template.name.equals(btps) ) {
								if ( m.lastDataPoint ) {
									btpsVal = formatBytes(m.lastDataPoint.value)
								}
							}
							if ( m.template.name.equals(prpm) ) {
								if ( m.lastDataPoint ) {
									prpmVal = "${m.lastDataPoint.value.round(2)}"
								}
							}
							if ( m.template.name.equals(ptpm) ) {
								if ( m.lastDataPoint ) {
									ptpmVal = "${m.lastDataPoint.value.round(2)}"
								}
							}
						}
						
						res << [row:row, resource:name, brps:brpsVal, btps:btpsVal, prpm:prpmVal , ptpm:ptpmVal]
						row +=1
					}
				}
			}
			else {
				res << [row:row, resource:'unavailable', brps:'unavailable', btps:'unavailable', prpm:'unavailable', ptpm:'unavailable']
			}
		}
		res
	}
	
	// Counts the total number of each Server
	// type that exists on a specific platform
	def getServerData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		PageControl pc = PageControl.PAGE_ALL
		def types = [:]
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				sMan.getServerTypesByPlatform(user, p.entityId.id, pc).each{ t ->
					if ( !res.server.contains(t.name) ) {
						def count = sDaoMan.findByPlatformAndType_orderName(p.entityId.id, t.id).size()
						res << [row:row, server:t.name, count:count]
						row += 1
					}
				}
				if ( res.server ) {
					res.sort { a,b ->
						a.server <=> b.server
					}
				}
			}
			else {
				res << [row:row, server:'unavailable', count:'unavailable']
			}
		}
		res
	}
	
	// Counts the total number of each Service
	// type that exists on a specific platform
	def getServiceData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def overlord = HQUtil.getOverlord()
		def allServices = []
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def servers = p.getViewableChildren(user)
				for ( srv in servers ) {
					def services = srv.getViewableChildren(user)
					allServices += services
				}
				for ( s in allServices ) {
					if ( !res.service.contains(s.prototype.name) ) {
						def count = 0
						def num = svcMan.getServicesByType(overlord, s.prototype.name, true)
						for ( n in num ) {
							if ( allServices.name.contains(n.name) ) {
								count += 1
							}
						}
						if ( num.name ) {
							res << [row:row, service:s.prototype.name, count:count]
							row += 1
						}
					}
				}
				if ( res.service ) {
					res.sort { a,b ->
						a.service <=> b.service
					}
				}
			}
			else {
				res << [row:row, service:'unavailable', count:'unavailable']
			}
		}
		res
	}
	
	// Find log information for servers on a platform
	// -- also finds log information for object spawner
	// services
	def getLogData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		PageControl pc = PageControl.PAGE_ALL
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				// find all of the platform's servers
				def servers = p.getViewableChildren(user)
				for ( s in servers ) {
					def location = "?"
					def logs = s.config.get('server.log_track.files')
					def install = s.config.get('installPath')
					
					// if install path and log directory value do
					// not exist, check to see if log tracking
					// is enabled
					if ( logs && logs.value && install.value ) {
						location = "${install.value}/${logs.value}"
					}
					// if log tracking is enabled, display that
					// the log location is not set
					else if ( s.config.get('server.log_track.enable') ) {
						location = "? (no log location set)"
					}
					// if log tracking is disabled, display that
					// log tracking is not enabled
					else {
						location = "? (log tracking not enabled)"
					}
					
					res << [row:row, server:s.prototype.name, location:location]
					row += 1
					
					// get log info for object spawner services using
					// the same approach as above
					if ( s.prototype.name.contains('Object Spawner') ) {
						def services = s.getViewableChildren(user)
						
						services.each{ svc ->
							location = "?"
							logs = svc.config.get('service.log_track.files')
							install = svc.config.get('installLocation')
							
							if ( logs && logs.value ) {
								location = logs.value
							}
							else if ( svc.config.get('service.log_track.enable') || svc.prototype.name.contains('Logical') ) {
								location = "? (no log location set)"
							}
							else {
								location = "? (log tracking not enabled)"
							}
							
							res << [row:row, server:svc.prototype.name, location:location]
							row += 1
						}
					}
				}
				if ( res.server ) {
					res.sort { a,b ->
						a.server <=> b.server
					}
				}
			}
			else {
				res << [row:row, server:'unavailable', location:'unavailable']
			}
		}
		res
	}
	
	// Finds the control history for the past
	// week of a platform. Change delta to change
	// the time period
	def getCtrlhisData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def delta = 7 * 24 * 60 * 60 * 1000
		def history = ctrlHisDaoMan.findByStartTime(now() - delta, true)
		
		for ( h in history ) {
			def resource
			def parent
			def list = false
			// find the parent platform of each resource
			// that has executed a control action
			switch ( h.entityType ) {
				case AppdefEntityConstants.APPDEF_TYPE_PLATFORM:
					resource = platMan.findPlatformById(h.entityId)
					if ( resource ) {
						parent = resource.name
					}
					break
				case AppdefEntityConstants.APPDEF_TYPE_SERVER:
					resource = sMan.getServerById(h.entityId)
					if ( resource  ) {
						parent = platMan.getPlatformByServer(user, resource.id).name
					}
					break
				case AppdefEntityConstants.APPDEF_TYPE_SERVICE:
					resource = svcMan.getServiceById(h.entityId)
					if ( resource ) {
						parent = platMan.getPlatformByService(user, resource.id).name
					}
					break
				//case AppdefEntityConstants.APPDEF_TYPE_GROUP:
				//	resource = gMan.findResourceGroupById(h.entityId)
				//	break
				default:
					resource = null
					parent = null
					break
			}
			
			// add to the map if the parent platform
			// of the resource is the selected platform
			if ( platform.equals(parent) ) {
				if ( resource ) {
					res << [row:row, resource:resource.name, dateTime:df.format(h.startTime), status:h.status, action:h.action, args:h.args, message:h.message]
					row += 1
				}
			}
		}
		res
	}
	
	// Finds the entire control action schedule
	// for all of resources for the selected platform
	def getCtrlschedData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def history = ctrlSchedDaoMan.findByFireTime(true)
		
		for ( h in history ) {
			def resource
			def parent
			def list = false
			// find the parent platform of each resource
			// that has a control action scheduled
			switch ( h.entityType ) {
				case AppdefEntityConstants.APPDEF_TYPE_PLATFORM:
					resource = platMan.findPlatformById(h.entityId)
					if ( resource ) {
						parent = resource.name
					}
					break
				case AppdefEntityConstants.APPDEF_TYPE_SERVER:
					resource = sMan.getServerById(h.entityId)
					if ( resource ) {
						parent = platMan.getPlatformByServer(user, resource.id).name
					}
					break
				case AppdefEntityConstants.APPDEF_TYPE_SERVICE:
					resource = svcMan.getServiceById(h.entityId)
					if ( resource ) {
						parent = platMan.getPlatformByService(user, resource.id).name
					}
					break
				//case AppdefEntityConstants.APPDEF_TYPE_GROUP:
				//	resource = gMan.findResourceGroupById(h.entityId)
				//	break
				default:
					resource = null
					parent = null
					break
			}
			
			// add to the map if the parent platform
			// of the resource is the selected platform
			if ( platform.equals(parent) ) {
				if ( resource ) {
					res << [row:row, resource:resource.name, dateTime:df.format(h.nextFireTime), action:h.action, scheduler:h.subject]
					row += 1
				}
			}
		}
		res
	}
	
	// Find the metrics for each of the file
	// mounts on a given system
	def getFilemountData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		PageControl pc = PageControl.PAGE_ALL
		def drpm = 'Disk Reads per Minute'
		def dwpm = 'Disk Writes per Minute'
		def tbf = 'Total Bytes Free'
		def up = 'Use Percent'
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				// get the type information for FileServer Mount
				// platform services
				def type = svcMan.findServiceTypeByName('FileServer Mount')
				if ( type ) {
					// once we have the FileServer Mount type, get the
					// platform services that exist of that type
					def services = svcMan.getServicesByPlatform(user, p.entityId.id, type.id, pc)
					for ( s in services ) {
						def name = s.name
						if ( s.name.contains(p.name) ) {
							name = s.name.minus(p.name)
						}
						
						def drpmVal = "?"
						def dwpmVal = "?"
						def tbfVal = "?"
						def upVal = "?"
						
						// the simple 'findByAliasAndID' method that we use to
						// get metrics for platforms doesn't work for servers
						// or services, so we're stuck querying the metrics
						// this way
						def metrics = mMan.findEnabledMeasurements(user, s.entityId, '')
						for ( m in metrics ) {
							if ( m.template.name.equals(drpm) ) {
								if ( m.lastDataPoint ) {
									drpmVal = "${m.lastDataPoint.value.round(2)}"
								}
							}
							if ( m.template.name.equals(dwpm) ) {
								if ( m.lastDataPoint ) {
									dwpmVal = "${m.lastDataPoint.value.round(2)}"
								}
							}
							if ( m.template.name.equals(tbf) ) {
								if ( m.lastDataPoint ) {
									tbfVal = formatBytes(m.lastDataPoint.value)
								}
							}
							if ( m.template.name.equals(up) ) {
								if ( m.lastDataPoint ) {
									upVal = "${(m.lastDataPoint.value * 100).round(2)}%"
								}
							}
						}
						
						res << [row:row, resource:name, drpm:drpmVal, dwpm:dwpmVal, tbf:tbfVal, up:upVal]
						row +=1
					}
				}
			} 
			else {
				res << [row:row, resource:'unavailable', drpm:'unavailable', dwpm:'unavailable', tbf:'unavailable', up:'unavailable']
			}
		}
		res
	}
	
	// Find the metrics for each of the nfs
	// mounts on a given system
	def getNfsmountData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		PageControl pc = PageControl.PAGE_ALL
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				// get the type information for RPC platform services
				def type = svcMan.findServiceTypeByName('RPC')
				if ( type ) {
					// once we have the RPC type, get the platform
					// services that exist of that type
					def services = svcMan.getServicesByPlatform(user, p.entityId.id, type.id, pc)
					for ( s in services ) {
						def name = s.name
						if ( s.name.contains(p.name) ) {
							name = s.name.minus(p.name)
						}
						def resTimeVal = "?"
						
						// the simple 'findByAliasAndID' method that we use to
						// get metrics for platforms doesn't work for servers
						// or services, so we're stuck querying the metrics
						// this way
						def m = mMan.findEnabledMeasurements(user, s.entityId, '').find{ e ->
							e.template.name == 'Response Time'
						}
						if ( m && m.lastDataPoint ) {
							resTimeVal = "${m.lastDataPoint.value}"
						}
						
						res << [row:row, resource:name, value:resTimeVal]
						row +=1
					}
				}
			}
			else {
				res << [row:row, resource:'unavailable', value:'unavailable']
			}
		}
		res
	}
	
	// Used to find the config options for the SAS
	// server methods below
	def getServerConfig(p, type, configOptions) {
		PageControl pc = PageControl.PAGE_ALL
		def count = 0
		def configVals = []
		def servers
		def exists = false

		if ( p ) {
			def platform = findPlatform(p)
			if ( platform ) {
				sMan.getServerTypesByPlatform(user, platform.entityId.id, pc).find{ t ->
					t.name.contains(type)
				}.each{ e ->
					// since there can only be one type of
					// each sas server per platform --
					// just set equal to all servers of
					// that type
					servers = sDaoMan.findByPlatformAndType_orderName(platform.entityId.id, e.id)
					
					// check to see if there are more
					// than one server for each type
					count = servers.size()
				}
				
				// loop through all servers that exist of that type
				for ( srv in servers ) {
					def s = rMan.findResource(srv.entityId)
					if ( s.prototype.name.contains(type) ) {
						if ( s.config ) {
							for ( o in configOptions ) {
								def config = s.config.get(o)
								if ( config ) {
									def description = o
									def property = o
									if ( config.description ) {
										description = config.description
										property = config.description
									}
									// if there are more than one server of
									// the same type, append the server onto
									// the property name to distinguish them
									if ( count > 1 ) {
										def name = s.name
										if ( s.name.contains ( platform.name ) ){
											name = s.name.minus(platform.name)
										}
										property = "${name} - ${description}"
									}
									configVals << [property:property, config:config.value]
								}
							}
						}
					}	
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		if ( configVals.property ) {
			configVals.sort { a,b ->
				a.property <=> b.property
			}
		}
		configVals
	}
	
	// Used to find the config options for the SAS
	// Object Spawner service methods below
	def getServiceConfig(p, serverType, type, configOptions) {
		PageControl pc = PageControl.PAGE_ALL
		def count = 0
		def configVals = []
		def servers
		def exists = false

		if ( p ) {
			def platform = findPlatform(p)
			if ( platform ) {
				sMan.getServerTypesByPlatform(user, platform.entityId.id, pc).find{ t ->
					t.name.contains(serverType)
				}.each{ e ->
					// since there can only be one type of
					// each sas server per platform --
					// just set equal to all servers of
					// that type
					servers = sDaoMan.findByPlatformAndType_orderName(platform.entityId.id, e.id)
					
					// find all services of the provided
					// type with the above server as the parent
					svcMan.getServiceTypesByServerType(user, e.id).find{ st ->
						st.name.contains(type)
					}.each{ se ->
						// check to see if there are more
						// than one service for each type
						count += svcDaoMan.findByType(se.id, true).size()
					}
				}
				
				// loop through all services that exist of that type
				for ( srv in servers ) {
					def server = rMan.findResource(srv.entityId)
					def services = server.getViewableChildren(user)
					for ( s in services ) {
						if ( s.prototype.name.contains(type) ) {
							if ( s.config ) {
								for ( o in configOptions ) {
									def config = s.config.get(o)
									if ( config ) {
										def description = o
										def property = o
										if ( config.description ) {
											description = config.description
											property = config.description
										}
										// if there are more than one service of
										// the same type, append the service onto
										// the property name to distinguish them
										if ( count > 1 ) {
											def name = s.name
											if ( s.name.contains ( platform.name ) ){
												name = s.name.minus(platform.name)
											}
											property = "${name} - ${description}"
										}
										configVals << [property:property, config:config.value]
									}
								}
							}
						}	
					}
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		if ( configVals.property ) {
			configVals.sort { a,b ->
				a.property <=> b.property
			}
		}
		configVals
	}
	
	// The below methods are for all of the SAS servers
	// and call one of the above two methods -- 
	// 'getServiceConfig' or 'getServerConfig'
	def getConnspawnData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['installPath', 'proc_query', 'iom_host', 'iom_port', 'resourceID', 'IOM_UUID', 'LongVersion']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def type = 'SAS Connect Spawner'
				def config = getServerConfig(p.name, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	def getMetaData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['installPath', 'proc_query', 'iom_host', 'iom_port', 'resourceID', 'IOM_UUID', 'LongVersion']
		
		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def type = 'SAS Metadata Server'
				def config = getServerConfig(p.name, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	def getOlapData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['installPath', 'proc_query', 'iom_host', 'iom_port', 'resourceID', 'IOM_UUID', 'LongVersion']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def type = 'SAS OLAP Server'
				def config = getServerConfig(p.name, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	def getObjspawnData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['installPath', 'proc_query', 'iom_host', 'iom_port', 'resourceID', 'IOM_UUID', 'LongVersion']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def type = 'SAS Object Spawner'
				def config = getServerConfig(p.name, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	def getStoredprocData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['context', 'installLocation', 'proc_query', 'resourceID', 'Version', 'LongVersion']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def serverType = 'SAS Object Spawner'
				def type = 'SAS Stored Process Server'
				def config = getServiceConfig(p.name, serverType, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	def getWorkspaceData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['context',  'installLocation', 'proc_query', 'resourceID', 'Version']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def serverType = 'SAS Object Spawner'
				def type = 'SAS Workspace Server'
				def config = getServiceConfig(p.name, serverType, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}

	def getPooledworkspaceData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['context',  'installLocation', 'proc_query', 'resourceID', 'Version']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def serverType = 'SAS Object Spawner'
				def type = 'SAS Pooled Workspace Server'
				def config = getServiceConfig(p.name, serverType, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}

	def getOsservicesData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['InstallLocation', 'ResourceID', 'Version']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def serverType = 'SAS Object Spawner'
				def type = 'SAS Operating System Services'
				def config = getServiceConfig(p.name, serverType, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	def getActivemqData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['installPath', 'process.query', 'jmx.url']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def type = 'ActiveMQ'
				def config = getServerConfig(p.name, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	def getTomcatData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['installPath', 'process.query', 'jmx.url', 'listen.ports']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def type = 'Apache Tomcat'
				def config = getServerConfig(p.name, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	def getTcruntimeData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def configOptions = ['installPath', 'ptql', 'jmx.url', 'catalina.base', 'catalina.home', 'listen.ports', 'program']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				def type = 'SpringSource tc Runtime'
				def config = getServerConfig(p.name, type, configOptions)
				
				for ( c in config ) {
					res << [row:row, property:c.property, config:c.config]
					row += 1
				}
			}
			else {
				res << [row:row, property:'unavailable', config:'unavailable']
			}
		}
		res
	}
	
	// Finds the license information for the
	// current deployment
	def getLicenseData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def prototype = ['SAS Application Server Tier']
		def configOptions = ['LicenseExpiration', 'LicenseTerminationWarning', 'LicenseTermination']
		
		// there is only one app server tier resource
		// per deployment, so the only one will be
		// found here
		def ps = rDaoMan.getResourcesByProtoTypeName(prototype)
		for ( p in ps ) {
			configOptions.each{ o ->
				def m = mDaoMan.findByAliasAndID(o, p)
				if ( m && m.lastDataPoint ) {
					def metric = m.lastDataPoint
					
					res << [row:row, property:m.template.name, value:metric]
					row += 1
				}
			}
		}
		res
	}

    // gets the custom properties for a resource
	def getCprops(resource) {
		//def overlord = Bootstrap.getBean(AuthzSubjectManager.class).overlordPojo
	    def aev = AppdefEntityID.newServerID(resource.id)
        def cprops = cpropMan.getDescEntries(aev)

        cprops
	}

	def getResourceData(resource, formats) {
		def row = 1
		def res = []

		def cprops = getCprops(resource)

		cprops.each{ c->
			res << [row:row, property:c.key, value:c.value]
			row +=1
		}

		def metrics = mMan.findEnabledMeasurements(user, resource.entityId, '')
		for ( m in metrics ) {
			if ( m && m.lastDataPoint ) {
				if ( !m.template.name.equals('Availability') ) {
					def ldp = m.lastDataPoint.value

					def format = formats.find{ f ->
						f.name == m.template.name
					}

					if ( format ) {
						if ( format.conv ) {
							if ( format.conv == 'time' ) {
								if ( format.base == 'sec' ) {
									ldp = ldp * 1000
								}
								
								ldp = formatTime(ldp)
							}
							else if ( format.conv == 'datetime' ) {
								ldp = df.format(ldp)
							}
							else if ( format.conv == 'bytes' ) {
								if ( format.base == 'kb' ) {
									ldp *= 1024
								}

								ldp = formatBytes(ldp)
							}
						}

						res << [row:row, property:format.name, value:ldp]
						row +=1
					}
					else {
						res << [row:row, property:m.template.name, value:ldp]
						row +=1
					}
				}
			}
		}

		if (!res){
			res << [row:row, property:'no data available', value:'no data available']
		}

		res
	}

	def getResourceObject(p, formats, resLev, etlType) {
		def res = []
		def datamartType = 'SAS Environment Manager Data Mart 9.4'
		PageControl pc = PageControl.PAGE_ALL

		if ( p ) {
			def platform = findPlatform(p)
			if (platform) {
				def prototype = sTypeDaoMan.findByName(datamartType)
				if ( prototype ) {
					def servers = sMan.getServersByPlatform(user, platform.entityId.id, prototype.id, true, pc)

					servers.each{ s->
						if ( resLev == 'datamart' ) {
							res = getResourceData(s, formats)
						}
						else if ( resLev == 'etl' ) {
							def etlPrototype = svcTypeDaoMan.findByName(etlType)
							if ( etlPrototype ) {
								def services = svcMan.getServicesByPlatform(user, platform.entityId.id, etlPrototype.id, pc)
								
								services.each{ svc->
									res = getResourceData(svc, formats)
								}
							}
						}
					}
				}
			}
		}

		if (!res){
			res << [row:1, property:'unavailable', value:'unavailable']
		}
		if ( res.property ) {
			res.sort { a,b ->
				a.property <=> b.property
			}
		}

		res
	}

	def getDatamartData(p) {
		def formats = [ [name:'ARTIFACTS Size', conv:'bytes', base:'b'],
						[name:'Database Mirror Size', conv:'bytes', base:'b'],
						[name:'Raw Data Size', conv:'bytes', base:'b'],
						[name:'Reporting Size', conv:'bytes', base:'b'],
						[name:'Total Reporting Size', conv:'bytes', base:'b'],
						[name:'Total Size', conv:'bytes', base:'b'] ]

		def res = getResourceObject(p, formats, 'datamart', '')

		res
	}

	def getAcmData(p) {
		def etlType = 'SAS Environment Manager Data Mart 9.4 ACM ETL Processing'

		def formats = [ [name:'Availability Extract Processing Time', conv:'time', base:'sec'],
						[name:'Database Mirror Processing Time', conv:'time', base:'sec'],
						[name:'Event Extract Processing Time', conv:'time', base:'sec'],
						[name:'Inventory Update Processing Time', conv:'time', base:'sec'],
						[name:'Last End Time', conv:'datetime', base:'milli'],
						[name:'Last Run Duration', conv:'time', base:'milli'],
						[name:'Last Start Time', conv:'datetime', base:'milli'],
						[name:'Metric Extract Processing Time', conv:'time', base:'sec'],
						[name:'Metric Extract Time Period', conv:'time', base:'sec'],
						[name:'Report Table Update Processing Time', conv:'time', base:'sec'] ]

		def res = getResourceObject(p, formats, 'etl', etlType)

		res
	}

	def getApmData(p) {
		def etlType = 'SAS Environment Manager Data Mart 9.4 APM ETL Processing'

		def formats = [ [name:'Archive Dataset Size', conv:'bytes', base:'kb'],
						[name:'Artifact Dataset Size', conv:'bytes', base:'kb'],
						[name:'Last End Time', conv:'datetime', base:'milli'],
						[name:'Last Run Duration', conv:'time', base:'milli'],
						[name:'Last Start Time', conv:'datetime', base:'milli'],
						[name:'Status Dataset Size', conv:'bytes', base:'kb'],
						[name:'Total Dataset Size', conv:'bytes', base:'kb'] ]

		def res = getResourceObject(p, formats, 'etl', etlType)

		res
	}
	

	// Executes 'top' and gathers resulting
	// data from each platform
	def getTopData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def tmpres = []
		def cmd = "top"
		def options = ['uptime', 'procStat', 'cpu', 'mem', 'swap', 'processes']
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				if ( getAvailability(p) ) {
					def aeid = p.entityId
					def output = p.getLiveData(user, cmd, new ConfigResponse())
					if ( output.errorMessage ) {
						log.error "Top Data Gathering Error: " + output.errorMessage
					}
					else if ( output && output.objectResult ) {
						String path = new File(".").canonicalPath
						def m = (path =~ /server-(.+)-EE/)
						if (m) {
						    def ver = m[0][1]
							def result = output.objectResult
							
							for ( x in result.processes ) {
								// a giant string of processes is returned
								// so we need to split each one apart by
								// the commas to separate the elements
								
								def r = x.toString().tokenize(",")
								
								def size = r[3]
								def sizeSub = size[-1]
								if ( sizeSub.equals("K") ) {
									def sizeMain = size[0..-2].toDouble()
									sizeMain /= 1024
									size = sizeMain.round(2) + "M"
								}
								else if ( sizeSub.equals("M") ) {
									size = size[0..-2].toDouble()
								}
								else if ( sizeSub.equals("G") ) {
									def sizeMain = size[0..-2].toDouble()
									size = sizeMain * 1024
								}
								else {
									size = size.toDouble()
								}
								
								def rss = r[4]
								def rssSub = rss[-1]
								if ( rssSub.equals("K") ) {
									def rssMain = rss[0..-2].toDouble()
									rssMain /= 1024
									rss = rssMain.round(2)
								}
								else if ( rssSub.equals("M") ) {
									rss = rss[0..-2].toDouble()
								}
								else if ( rssSub.equals("G") ) {
									def rssMain = rss[0..-2].toDouble()
									rss = rssMain * 1024
								}
								else {
									rss = rss.toDouble()
								}
								
								def share = r[5]
								def shareSub = share[-1]
								if ( shareSub.equals("K") ) {
									def shareMain = share[0..-2].toDouble()
									shareMain /= 1024
									share = shareMain.round(2)
								}
								else if ( shareSub.equals("M") ) {
									share = share[0..-2].toDouble()
								}
								else if ( shareSub.equals("G") ) {
									def shareMain = share[0..-2].toDouble()
									share = shareMain * 1024
								}
								else if ( !share.equals("-") ) {
									share = share.toDouble()
								}
									
								
								if (ver == '5.0.0') {
									tmpres << [pid:r[0], user:r[1], stime:r[2], size:size, rss:rss, share:share, state:r[6], time:r[7], cpu:r[8], cmd:r[9]]
								}
								else if (ver == '5.8.0') {
									tmpres << [pid:r[0], user:r[1], stime:r[2], size:size, rss:rss, share:share, state:r[6], time:r[7], cpu:r[8], cmd:r[10]]
								}
							}
						}
						else {
							tmpres << [pid:'', user:'', stime:'', size:'', rss:'', share:'', state:'', time:'', cpu:'', cmd:'Unable to detect Environment Manager version.']
						}
					}
					if ( tmpres.rss ) {
						tmpres.sort { a,b ->
							b.rss <=> a.rss
						}
					}
					// only store the first 100 lines of the
					// output. the entire thing is too big
					for ( t in tmpres ) {
						if ( row < 101 ) {
							res << [row:row, pid:t.pid, user:t.user, stime:t.stime, size:t.size, rss:t.rss, share:t.share, state:t.state, time:t.time, cpu:t.cpu, cmd:t.cmd]
							row +=1
						}
						else {
							break
						}
					}
				}
			}
			else {
				res << [row:row, pid:'unconfigured', user:'unconfigured', stime:'unconfigured', size:'unconfigured', rss:'unconfigured', share:'unconfigured', state:'unconfigured', time:'unconfigured', cpu:'unconfigured', cmd:'unconfigured']
			}
		}
		res
	}
	
	// Executes 'df' and gathers resulting
	// data from each platform
	def getDfData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 0
		def res = []
		def matches = []
		def cmd = "df"
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p && getAvailability(p) ) {
				def aeid = p.entityId
				def output = p.getLiveData(user, cmd, new ConfigResponse())
				if ( output.errorMessage ) {
					log.error "Df Data Gathering Error: " + output.errorMessage
				}
				else if ( output && output.XMLResult ) {
					def response = output.XMLResult
					if ( output.objectResult ) {
						def result = output.objectResult
						if ( result.fileSystems ) {
							for ( f in result.fileSystems ) {
								// the resulting data is in the form of a
								// very messy string of xml output -- we need
								// to parse this to get out the data that we
								// want
							
								def devName = response =~ /(?s)<devName>(.*?)<\/devName>/
								def sysTypeName = response =~ /(?s)<sysTypeName>(.*?)<\/sysTypeName>/
								def typeName = response =~ /(?s)<typeName>(.*?)<\/typeName>/
								def finalType = sysTypeName[row][1] + "/" + typeName[row][1]
								def name = devName[row][1]
								def size = '?'
								def used = '?'
								def avail = '?'
								def usePerc = '?'
								
								if ( f.stat ) {
									if ( f.stat.total ) {
										size = (double)(f.stat.total / 1048576)
										size = size.round(2)
									}
									if ( f.stat.used ) {
										used = (double)(f.stat.used / 1048576)
										used = used.round(2)
									}
									if ( f.stat.avail ) {
										avail = (double)(f.stat.avail / 1048576)
										avail = avail.round(2)
									}
									if ( f.stat.usePercent ) {
										usePerc = (f.stat.usePercent * 100).round(1) + "%"
									}
								}
								
								res << [row:row, name:name, size:size, used:used, avail:avail, usePerc:usePerc, mount:f.config, type:finalType]
								row +=1
							}
						}
					}
					if ( res.name ) {
						res.sort { a,b ->
							a.name <=> b.name
						}
					}
				}
			}
			else {
				res << [row:row, name:'unavailable', size:'unavailable', used:'unavailable', avail:'unavailable', usePerc:'unavailable', mount:'unavailable', type:'unavailable']
			}
		}
		res
	}
	
	// Executes 'who' and gathers resulting
	// data from each platform
	def getWhoData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def cmd = "who"
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p && getAvailability(p) ) {
				def aeid = p.entityId
				def output = p.getLiveData(user, cmd, new ConfigResponse())
				if ( output.errorMessage ) {
					log.error "Who Data Gathering Error: " + output.errorMessage
				}
				else if ( output && output.objectResult ) {
					def result = output.objectResult
					
					for ( r in result ) {
						def time = new Date(r.time * 1000)
						
						res << [row:row, user:r.user, device:r.device, time:time, host:r.host]
						row +=1
					}
				}
			}
			else {
				res << [row:row, user:'unavailable', device:'unavailable', time:'unavailable', host:'unavailable']
			}
		}
		res
	}
	
	// Find all snapshots that currently exist in
	// the filesystem and sort them by most recently
	// modified
	def getSnapshotData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		def overlord = HQUtil.getOverlord()
		def types = [:]
		
		def now = new Date()
		// set the time period to check for snapshots
		// within to 60 days
		def checkTime = {file -> now - new Date(file.lastModified()) < 60}
		
		// find all snapshots within the above time period
		// sorted by the last modied files first
		def newest = new File('./hq-engine/hq-server/webapps/ROOT/hqu/emi-snapshot/public/snapshots').listFiles().toList().findAll(checkTime).sort{ -it.lastModified() }
		//def snapshots = false
		if ( newest ) {
			for ( n in newest ) {
				if ( n.isFile() && n.lastModified() ) { 
					def formatFile = n.name
					def modified = new Date(n.lastModified()).format('yyyy-MM-dd HH:mm:ss')
					def latestSize = (double)(n.length() / 1048576)
					
					// create a link to be used in the snapshots
					// table so the user can instantly view
					// created snapshots
					def fileLink = linkTo(formatFile, [asset:"snapshots/" + formatFile])
					
					res << [row:row, name:fileLink, size:latestSize.round(2), modified:modified]
					row += 1
				}
			}
		}
		res
	}
	
	// Find all events that have occurred on the
	// given platform or any children of that
	// platform during the past week
	def getEventData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		PageControl pc = PageControl.PAGE_ALL
		def delta = 7 * 24 * 60 * 60 * 1000
		Collection<String> eTypes = Collections.EMPTY_LIST
		// code 10 means ALL types
		def eventEnum = EventLogStatus.findByCode(10)
		// the '9999' here is the max amount to retrieve
		def pInfo = PageInfo.create(0, 9999, EventLogSortField.DATE, false)  
		def resources = []
		def servers = []
		def services = []
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				resources += p.id
				servers = p.getViewableChildren(user)
				for ( s in servers ) {
					resources += s.id
					resources += s.getViewableChildren(user).id
				}
				
				// find event 'logs'
				def logVar = eMan.findLogs( user, now() - delta, now(), pInfo, eventEnum, null, null)
				
				for ( lv in logVar ) {
					if ( resources.contains(lv.resource.id) ) {
						def time = new Date(lv.eventLog.timestamp).format('yyyy-MM-dd HH:mm:ss')
						// convert the given status (eg ALR) to something
						// more readable for the snapshots and the table
						def status = checkStatus(lv.eventLog.status)
						
						res << [row:row, date:time, status:status, resource:lv.resource.name, subject:lv.eventLog.subject, detail:lv.eventLog.detail]
						row +=1
					}
				}
			}
			else {
				res << [row:row, date:'unavailable', status:'unavailable', resource:'unavailable', subject:'unavailable', detail:'unavailable']
			}
		}
		res
	}
	
	// Find all alerts that have occurred on the
	// given platform or any children of that
	// platform during the past week
	def getAlertData(platform) {
		// each row must have unique id and CANNOT
		// be null -- breaks if rowId=null
		def row = 1
		def res = []
		PageControl pc = PageControl.PAGE_ALL
		def delta = 7 * 24 * 60 * 60 * 1000
		Collection<String> eTypes = Collections.EMPTY_LIST
		// code 10 means ALL types -- there is no code for ALR
		def eventEnum = EventLogStatus.findByCode(10)
		// the '9999' here is the max amount to retrieve
		def pInfo = PageInfo.create(0, 9999, EventLogSortField.DATE, false)  
		def resources = []
		def servers = []
		def services = []
		def exists = false

		if ( platform ) {
			def p = findPlatform(platform)
			if ( p ) {
				resources += p.id
				servers = p.getViewableChildren(user)
				for ( s in servers ) {
					resources += s.id
					resources += s.getViewableChildren(user).id
				}
				
				// find event 'logs'
				def logVar = eMan.findLogs( user, now() - delta, now(), pInfo, eventEnum, null, null)
				
				for ( lv in logVar ) {
					if ( resources.contains(lv.resource.id) ) {
						// only take the event logs that are of 
						// status ALR -- we can auto-convert the
						// status to Alert in the final map
						if ( lv.eventLog.status.equals('ALR') ) {
							def time = new Date(lv.eventLog.timestamp).format('yyyy-MM-dd HH:mm:ss')
							
							res << [row:row, date:time, status:'Alert', resource:lv.resource.name, subject:lv.eventLog.subject, detail:lv.eventLog.properties]
							row +=1
						}
					}
				}
			}
			else {
				res << [row:row, date:'unavailable', status:'unavailable', resource:'unavailable', subject:'unavailable', detail:'unavailable']
			}
		}
		res
	}
	
	// Find all of the plugins and their
	// version numbers
	def getPluginData(platform) {
		def row = 1
		def res = []
		def plugins = pluginMan.getAllPlugins()
		
		for ( p in plugins ) {
			res << [row:row, name:p.name, version:p.version]
			row +=1
		}
		if ( res.name ) {
			res.sort { a,b ->
				a.name <=> b.name
			}
		}
		res
	}
	
	// The methods below are used as wrappers for
	// creating a snapshot report -- they set up
	// the headers for the 'tables' in the snapshot
	// then gather the information for each platform
	
	def snapHardwareData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
				res << [property:'-----', config:'-----']
				def data = getHardwareData(p.name)
				
				data.each{ d ->
					if ( !d.property.equals('') ) {
						res << [property:d.property, config:d.config]		
					}
				}
				res << [property:'', config:'']
			}
		}
		
		res
	}
	
	def snapCpuData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, resource:"${localeBundle.processor}", value:"${localeBundle.cpuUsage}"]
				res << [resource:'-----', value:'-----']
				def data = getCpuData(p.name)
				
				data.each{ d ->
					res << [resource:d.resource, value:d.value]		
				}
				res << [resource:'', value:'']
			}
		}
		
		res
	}
	
	def snapSystemData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
				res << [property:'-----', config:'-----']
				def data = getSystemData(p.name)
				
				data.each{ d ->
					res << [property:d.property, config:d.config]		
				}
				res << [property:'', config:'']
			}
		}
		
		res
	}
	
	def snapNetworkData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
				res << [property:'-----', config:'-----']
				def data = getNetworkData(p.name)
				
				data.each{ d ->
					res << [property:d.property, config:d.config]		
				}
				res << [property:'', config:'']
			}
		}
		
		res
	}
	
	def snapNetintData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, resource:"${localeBundle.netInt}", brps:"${localeBundle.brps}", btps:"${localeBundle.btps}", prpm:"${localeBundle.prpm}", ptpm:"${localeBundle.ptpm}"]
				res << [resource:'-----', brps:'-----', btps:'-----', prpm:'-----', ptpm:'-----']
				def data = getNetintData(p.name)
				
				data.each{ d ->
					res << [resource:d.resource, brps:d.brps, btps:d.btps, prpm:d.prpm, ptpm:d.ptpm]		
				}
				res << [resource:'', brps:'', btps:'', prpm:'', ptpm:'']
			}
		}
		
		res
	}
	
	def snapFilemountData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, resource:"${localeBundle.mount}", drpm:"${localeBundle.drpm}", dwpm:"${localeBundle.dwpm}", tbf:"${localeBundle.tbf}", up:"${localeBundle.up}"]
				res << [resource:'-----', drpm:'-----', dwpm:'-----', tbf:'-----', up:'-----']
				def data = getFilemountData(p.name)
				
				data.each{ d ->
					res << [resource:d.resource, drpm:d.drpm, dwpm:d.dwpm, tbf:d.tbf, up:d.up]		
				}
				res << [resource:'', drpm:'', dwpm:'', tbf:'', up:'']
			}
		}
		
		res
	}
	
	def snapNfsmountData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, resource:"${localeBundle.mount}", value:"${localeBundle.resTime}"]
				res << [resource:'-----', value:'-----']
				def data = getNfsmountData(p.name)
				
				data.each{ d ->
					res << [resource:d.resource, value:d.value]		
				}
				res << [resource:'', value:'']
			}
		}
		
		res
	}
	
	def snapServerData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, server:"${localeBundle.name}", count:"${localeBundle.count}"]
				res << [server:'-----', count:'-----']
				def data = getServerData(p.name)
				
				data.each{ d ->
					res << [server:d.server, count:d.count]		
				}
				res << [server:'', count:'']
			}
		}
		
		res
	}
	
	def snapServiceData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, service:"${localeBundle.name}", count:"${localeBundle.count}"]
				res << [service:'-----', count:'-----']
				def data = getServiceData(p.name)
				
				data.each{ d ->
					res << [service:d.service, count:d.count]		
				}
				res << [service:'', count:'']
			}
		}
		
		res
	}
	
	def snapLogData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, server:"${localeBundle.server}", location:"${localeBundle.logLocation}"]
				res << [server:'-----', location:'-----']
				def data = getLogData(p.name)
				
				data.each{ d ->
					res << [server:d.server, location:d.location]		
				}
				res << [server:'', location:'']
			}
		}
		
		res
	}
	
	def snapCtrlhisData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, resource:"${localeBundle.resource}", dateTime:"${localeBundle.dateTime}", status:"${localeBundle.status}", action:"${localeBundle.action}", args:"${localeBundle.args}", message:"${localeBundle.message}"]
				res << [resource:'-----', dateTime:'-----', status:'-----', action:'-----', args:'-----', message:'-----']
				def data = getCtrlhisData(p.name)
				
				if ( data ) {
					data.each{ d ->
						res << [resource:d.resource, dateTime:d.dateTime, status:d.status, action:d.action, args:d.args, message:d.message]	
					}
				}
				else {
					res << [resource:'No control history information found for this platform...', dateTime:'', status:'', action:'', args:'', message:'']
				}
				res << [resource:'', dateTime:'', status:'', action:'', args:'', message:'']
			}
		}
		
		res
	}

	def snapCtrlschedData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, resource:"${localeBundle.resource}", dateTime:"${localeBundle.dateTime}", action:"${localeBundle.action}", scheduler:"${localeBundle.scheduler}"]
				res << [resource:'-----', dateTime:'-----', action:'-----', scheduler:'-----']
				def data = getCtrlschedData(p.name)
				
				if ( data ) {
					data.each{ d ->
						res << [resource:d.resource, dateTime:d.dateTime, action:d.action, scheduler:d.scheduler]
					}
				}
				else {
					res << [resource:'No control schedule information found for this platform...', dateTime:'', action:'', scheduler:'']
				}
				res << [resource:'', dateTime:'', action:'', scheduler:'']
			}
		}
		
		res
	}
	
	def snapConnspawnData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getConnspawnData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']
					
					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}
	
	def snapMetaData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getMetaData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']
					
					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}
	
	def snapOlapData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getOlapData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']

					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}
	
	def snapObjspawnData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getObjspawnData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']
					
					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}
	
	def snapStoredprocData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getStoredprocData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']
					
					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}
	
	def snapWorkspaceData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getWorkspaceData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']

					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}

	def snapPooledworkspaceData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getPooledworkspaceData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']

					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}

	def snapOsservicesData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getOsservicesData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']

					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}
	
	def snapActivemqData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getActivemqData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']
					
					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}
	
	def snapTomcatData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getTomcatData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']
					
					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}
	
	def snapTcruntimeData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def data = getTcruntimeData(p.name)
				
				if ( data ) {
					def name = '-' + p.name + '-'
					res << [platform:name, property:"${localeBundle.property}", config:"${localeBundle.value}"]
					res << [property:'-----', config:'-----']
					
					data.each{ d ->
						if ( !d.property.equals('') ) {
							res << [property:d.property, config:d.config]		
						}
					}
					res << [property:'', config:'']
				}
			}
		}
		
		res
	}
	
	def snapLicenseData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, property:"${localeBundle.property}", value:"${localeBundle.value}"]
				res << [property:'-----', value:'-----']
				def data = getLicenseData(p.name)
				
				data.each{ d ->
					res << [property:d.property, value:d.value]		
				}
				res << [property:'', value:'']
			}
		}
		
		res
	}
	
	def snapTopData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, pid:"${localeBundle.pid}", user:"${localeBundle.owner}", stime:"${localeBundle.stime}", size:"${localeBundle.size}", rss:"${localeBundle.rss}", share:"${localeBundle.share}", state:"${localeBundle.state}", time:"${localeBundle.time}", cpu:"${localeBundle.cpu}", cmd:"${localeBundle.cmd}"]
				res << [pid:'-----', user:'-----', stime:'-----', size:'-----', rss:'-----', share:'-----', state:'-----', time:'-----', cpu:'-----', cmd:'-----']
				def data = getTopData(p.name)
				
				data.each{ d ->
					res << [pid:d.pid, user:d.user, stime:d.stime, size:d.size, rss:d.rss, share:d.share, state:d.state, time:d.time, cpu:d.cpu, cmd:d.cmd]
				}
				res << [pid:'', user:'', stime:'', size:'', rss:'', share:'', state:'', time:'', cpu:'', cmd:'']
			}
		}
		
		res
	}
	
	def snapDfData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def platName = '-' + p.name + '-'
				res << [platform:platName, name:"${localeBundle.name}", size:"${localeBundle.sizegb}", used:"${localeBundle.usedgb}", avail:"${localeBundle.availgb}", usePerc:"${localeBundle.usePerc}", mount:"${localeBundle.mount}", type:"${localeBundle.type}"]
				res << [name:'-----', size:'-----', used:'-----', avail:'-----', usePerc:'-----', mount:'-----', type:'-----']
				def data = getDfData(p.name)
				
				data.each{ d ->
					res << [name:d.name, size:d.size, used:d.used, avail:d.avail, usePerc:d.usePerc, mount:d.mount, type:d.type]		
				}
				res << [name:'', size:'', used:'', avail:'', usePerc:'', mount:'', type:'']
			}
		}
		
		res
	}
	
	def snapWhoData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, user:"${localeBundle.user}", time:"${localeBundle.time}", device:"${localeBundle.device}", host:"${localeBundle.host}"]
				res << [user:'-----', time:'-----', device:'-----', host:'-----']
				def data = getWhoData(p.name)
				
				data.each{ d ->
					res << [user:d.user, time:d.time, device:d.device, host:d.host]		
				}
				res << [user:'', time:'', device:'', host:'']
			}
		}
		
		res
	}

	def snapDatamartData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, property:"${localeBundle.property}", value:"${localeBundle.value}"]
				res << [property:'-----', value:'-----']
				def data = getDatamartData(p.name)
				
				data.each{ d ->
					res << [property:d.property, value:d.value]
				}
				res << [property:'', value:'']
			}
		}
		
		res
	}

	def snapAcmData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, property:"${localeBundle.property}", value:"${localeBundle.value}"]
				res << [property:'-----', value:'-----']
				def data = getAcmData(p.name)
				
				data.each{ d ->
					res << [property:d.property, value:d.value]
				}
				res << [property:'', value:'']
			}
		}
		
		res
	}

	def snapApmData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, property:"${localeBundle.property}", value:"${localeBundle.value}"]
				res << [property:'-----', value:'-----']
				def data = getApmData(p.name)
				
				data.each{ d ->
					res << [property:d.property, value:d.value]
				}
				res << [property:'', value:'']
			}
		}
		
		res
	}

	def snapEventData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, date:"${localeBundle.dateTime}", status:"${localeBundle.status}", resource:"${localeBundle.resource}", subject:"${localeBundle.subject}", detail:"${localeBundle.detail}"]
				res << [date:'-----', status:'-----', resource:'-----', subject:'-----', detail:'-----']
				def data = getEventData(p.name)
				
				if ( data ) {
					data.each{ d ->
						res << [date:d.date, status:d.status, resource:d.resource, subject:d.subject, detail:d.detail]		
					}
				}
				else {
					res << [date:'No event information found for this platform...', status:'', resource:'', subject:'', detail:'']
				}
				res << [date:'', status:'', resource:'', subject:'', detail:'']
			}
		}
		
		res
	}
	
	def snapAlertData() {
		def platforms = resourceHelper.findAllPlatforms()
		def unknown = "?"
		def res = []
		
		for ( p in platforms ) {
			if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
				def name = '-' + p.name + '-'
				res << [platform:name, date:"${localeBundle.dateTime}", status:"${localeBundle.status}", resource:"${localeBundle.resource}", subject:"${localeBundle.subject}", detail:"${localeBundle.detail}"]
				res << [date:'-----', status:'-----', resource:'-----', subject:'-----', detail:'-----']
				def data = getAlertData(p.name)
				
				if ( data ) {
					data.each{ d ->
						res << [date:d.date, status:d.status, resource:d.resource, subject:d.subject, detail:d.detail]		
					}
				}
				else {
					res << [date:'No alert information found for this platform...', status:'', resource:'', subject:'', detail:'']
				}
				res << [date:'', status:'', resource:'', subject:'', detail:'']
			}
		}
		
		res
	}
	
	def getPlatformList() {
		def res = []
		def overlord = HQUtil.getOverlord()
		def platforms = resourceHelper.findAllPlatforms()
		
		for ( p in platforms ) {
			if ( p ) {
				if ( !p.prototype.name.equals('SAS Application Server Tier') ) {
					res << [platform:p.name]
				}
			}
		}
		res
	}
	
	def getServerList() {
		def res = []
		def overlord = HQUtil.getOverlord()
		PageControl pc = PageControl.PAGE_ALL
		def servers = resourceHelper.findAllServers()
		
		for ( s in servers ) {
			if ( s.prototype.name.contains('Object Spawner') ) {
				def services = svcMan.getServicesByServer(user, s.entityId.id, pc)
				for ( svc in services ) {
					res << [server:svc.name]
				}
			}
			else {
				res << [server:s.prototype.name]
			}
		}
		res
	}
	
	// The below methods use the Dojo Utility
	// to set up the tables for being displayed
	// in the main UI

	def hardwareData(params) {
		DojoUtil.processTableRequest(HARDWARE_SCHEMA, params)
	}
	
	def cpuData(params) {
		DojoUtil.processTableRequest(CPU_SCHEMA, params)
	}
	
	def systemData(params) {
		DojoUtil.processTableRequest(SYSTEM_SCHEMA, params)
	}
	
	def networkData(params) {
		DojoUtil.processTableRequest(NETWORK_SCHEMA, params)
	}
	
	def netintData(params) {
		DojoUtil.processTableRequest(NETINT_SCHEMA, params)
	}
	
	def filemountData(params) {
		DojoUtil.processTableRequest(FILEMOUNT_SCHEMA, params)
	}
	
	def nfsmountData(params) {
		DojoUtil.processTableRequest(NFSMOUNT_SCHEMA, params)
	}
	
	def serverData(params) {
        DojoUtil.processTableRequest(SERVER_SCHEMA, params)
    }
	
	def serviceData(params) {
        DojoUtil.processTableRequest(SERVICE_SCHEMA, params)
    }
	
	def logData(params) {
		DojoUtil.processTableRequest(LOG_SCHEMA, params)
	}
	
	def ctrlhisData(params) {
		DojoUtil.processTableRequest(CTRLHIS_SCHEMA, params)
	}
	
	def ctrlschedData(params) {
		DojoUtil.processTableRequest(CTRLSCHED_SCHEMA, params)
	}
	
	def connspawnData(params) {
		DojoUtil.processTableRequest(CONNSPAWN_SCHEMA, params)
	}
	
	def metaData(params) {
		DojoUtil.processTableRequest(META_SCHEMA, params)
	}
	
	def olapData(params) {
		DojoUtil.processTableRequest(OLAP_SCHEMA, params)
	}
	
	def objspawnData(params) {
		DojoUtil.processTableRequest(OBJSPAWN_SCHEMA, params)
	}
	
	def storedprocData(params) {
		DojoUtil.processTableRequest(STOREDPROC_SCHEMA, params)
	}
	
	def workspaceData(params) {
		DojoUtil.processTableRequest(WORKSPACE_SCHEMA, params)
	}
	
	def pooledworkspaceData(params) {
		DojoUtil.processTableRequest(POOLEDWORKSPACE_SCHEMA, params)
	}

	def osservicesData(params) {
		DojoUtil.processTableRequest(OSSERVICES_SCHEMA, params)
	}

	def activemqData(params) {
		DojoUtil.processTableRequest(ACTIVEMQ_SCHEMA, params)
	}
	
	def tomcatData(params) {
		DojoUtil.processTableRequest(TOMCAT_SCHEMA, params)
	}
	
	def tcruntimeData(params) {
		DojoUtil.processTableRequest(TCRUNTIME_SCHEMA, params)
	}
	
	def licenseData(params) {
		DojoUtil.processTableRequest(LICENSE_SCHEMA, params)
	}
	
	def topData(params) {
		DojoUtil.processTableRequest(TOP_SCHEMA, params)
	}
	
	def dfData(params) {
		DojoUtil.processTableRequest(DF_SCHEMA, params)
	}
	
	def whoData(params) {
		DojoUtil.processTableRequest(WHO_SCHEMA, params)
	}
	
	def snapshotData(params) {
		DojoUtil.processTableRequest(SNAPSHOT_SCHEMA, params)
	}
	
	def eventData(params) {
		DojoUtil.processTableRequest(EVENT_SCHEMA, params)
	}
	
	def alertData(params) {
		DojoUtil.processTableRequest(ALERT_SCHEMA, params)
	}

	def datamartData(params) {
		DojoUtil.processTableRequest(DATAMART_SCHEMA, params)
	}

	def acmData(params) {
		DojoUtil.processTableRequest(ACM_SCHEMA, params)
	}

	def apmData(params) {
		DojoUtil.processTableRequest(APM_SCHEMA, params)
	}
	
	// This method gathers random system stats and its
	// contents are used in the actual snapshot --
	// separated from the snapshot method to maintain
	// cleanliness
	def getSystemStats(params) {
        def s = Humidor.instance.sigar
        def dateFormat = DateFormat.getDateTimeInstance()
        
        def sysMem   = s.mem
        def sysSwap  = s.swap
        def pid      = s.pid
        def procFds  = 'unknown'
        def procMem  = s.getProcMem(pid)
        def procCpu  = s.getProcCpu(pid)
        def procTime = s.getProcTime(pid)
        def runtime  = Runtime.runtime
        def procEnv  = 'unknown'
        def cmdLine  = 'unknown'
            
        try {
            procFds = s.getProcFd(pid).total
        } catch(Exception e) {
        }

        try {
            procEnv = s.getProcEnv('$$')
        } catch(Exception e) {
        }

        try {
            cmdLine = s.getProcArgs('$$')
        } catch(Exception e) {
        }

        def free
        def used
        if ((sysMem.free != sysMem.actualFree ||
            (sysMem.used != sysMem.actualUsed))) {
            free = sysMem.actualFree
            used = sysMem.actualUsed
        } else {
            free = sysMem.free
            used = sysMem.used
        }
		
		return [totalMem:      formatBytes(sysMem.total),
                usedMem:       formatBytes(used),
                freeMem:       formatBytes(free),
				totalSwap:     formatBytes(sysSwap.total),
                usedSwap:      formatBytes(sysSwap.used),
                freeSwap:      formatBytes(sysSwap.free),
                pid:           pid,
                procStartTime: dateFormat.format(procTime.startTime),
                procOpenFds:   procFds,
                procMemSize:   formatBytes(procMem.size),
                procMemRes:    formatBytes(procMem.resident),
                procMemShare:  formatBytes(procMem.share),
                procCpu:       formatPercent(procCpu.percent, runtime.availableProcessors()),
                cmdLine:	   cmdLine,
                fqdn:          s.getFQDN(),
                procEnv: 	   procEnv,
                cpuInfos:      s.cpuInfoList,
		]
	}
	
	// This is the snapshot report method -- this
	// method gathers calls all necessary methods
	// and gathers all required info that is displayed
	// in the snapshot reports
	def snapshotReport(params) {
		log.info "Creating snapshot of environment..."
		
        def dateFormat  = DateFormat.dateTimeInstance
        def inventoryPager  = PageInfo.getAll(ResourceSortField.NAME, true)
		HttpServletRequest request = invokeArgs.request
		def response = request.getQueryString()
		def url = request.getRequestURL() + '?' + response
		def data = url.toURL().text
		
		// The variables from the checkboxes in the UI
		// are passed to the snapshot through URL parameters
		// -- parse them into a map here
		def map = response.split('&').inject([:]) { map, kv ->
			def (key, value) = kv.split('=').toList()
			if ( value != null ) {
				map[key] = value
			}
			map
		}
		
        def locals = [
			map:			  map,
			response:		  response,
            guid:             sConfMan.getGUID(),
            numCpu:           Runtime.runtime.availableProcessors(),
            reportTime:       dateFormat.format(System.currentTimeMillis()),
            userName:         user.fullName,
            numPlatforms:     resourceHelper.find(count:'platforms'),
            numCpus:          resourceHelper.find(count:'cpus'),            
            numAgents:        agentHelper.find(count:'agents'),
            numActiveAgents:  agentHelper.find(count:'activeAgents'),            
            numServers:       resourceHelper.find(count:'servers'),
            numServices:      resourceHelper.find(count:'services'),
            numApplications:  resourceHelper.find(count:'applications'),
            numRoles:         resourceHelper.find(count:'roles'),
            numUsers:         resourceHelper.find(count:'users'),
            numAlertDefs:     resourceHelper.find(count:'alertDefs'),
            numResources:     resourceHelper.find(count:'resources'),
            numResourceTypes: resourceHelper.find(count:'resourceTypes'),
            numGroups:        resourceHelper.find(count:'groups'),            
            numEscalations:   resourceHelper.find(count:'escalations'),
            numActiveEscalations:  resourceHelper.find(count:'activeEscalations'),
            hqVersion:        prodBossMan.version,
			schemaVersion:    sConfMan.config.getProperty('CAM_SCHEMA_VERSION'),
			inventoryData:    getInventoryData(),
			platformList: 	  platformList,
			serverList:		  serverList,
			dataFmt:	 	  dataFmt,
			ctrlHisDataFmt:	  ctrlHisDataFmt,
			ctrlSchedDataFmt: ctrlSchedDataFmt,
			netIntDataFmt:	  netIntDataFmt,
			fileMountDataFmt: fileMountDataFmt,
			nfsMountDataFmt:  nfsMountDataFmt,
			hardwareData:	  snapHardwareData(),
			cpuData:		  snapCpuData(),
			systemData:		  snapSystemData(),
			networkData:	  snapNetworkData(),
			netintData:	  	  snapNetintData(),
			filemountData:	  snapFilemountData(),
			nfsmountData:	  snapNfsmountData(),
			serverData:		  snapServerData(),
			serviceData:	  snapServiceData(),
			logData:		  snapLogData(),
			ctrlhisData:	  snapCtrlhisData(),
			ctrlschedData:	  snapCtrlschedData(),
			connspawnData:	  snapConnspawnData(),
			metaData:	  	  snapMetaData(),
			olapData:	  	  snapOlapData(),
			objspawnData:	  snapObjspawnData(),
			storedprocData:	  snapStoredprocData(),
			workspaceData:	  snapWorkspaceData(),
			pooledworkspaceData: snapPooledworkspaceData(),
			osservicesData:	  snapOsservicesData(),
			activemqData:	  snapActivemqData(),
			tomcatData:	  	  snapTomcatData(),
			tcruntimeData:	  snapTcruntimeData(),
			licenseData:	  snapLicenseData(),
			pluginData:		  getPluginData(),
			datamartData:	  snapDatamartData(),
			acmData: 		  snapAcmData(),
			apmData: 		  snapApmData()
        ] + getSystemStats([:])
        
		if ( map.live.equals("true") ) {
			locals += [ topDataFmt: topDataFmt,
						dfDataFmt:  dfDataFmt,
						whoDataFmt: whoDataFmt,
						topData: snapTopData(),
						dfData:  snapDfData(),
						whoData: snapWhoData() ]
		}
		if ( map.events.equals("true") ) {
			locals += [	eventDataFmt: eventDataFmt,
						eventData: snapEventData() ]
		}
		if ( map.alerts.equals("true") ) {
			locals += [	alertData: snapAlertData() ]
		}
		
        render(locals: locals)
		
		log.info "Environment snapshot has been created."
    }
	
	// This is the index method -- this
	// method gathers calls all necessary methods
	// and gathers all required info that is displayed
	// in the main UI
	def index(params) {
		log.info "Loading table info..."
		def pastDate = use( groovy.time.TimeCategory ) {
			new Date() - 45.seconds
		}
		println pastDate
		
		render(locals: [
			pastDate:		  pastDate,
			platformList: 	  platformList,
			serverList:		  serverList,
			hardwareSchema:   HARDWARE_SCHEMA,
			cpuSchema:		  CPU_SCHEMA,
			systemSchema: 	  SYSTEM_SCHEMA,
			networkSchema:	  NETWORK_SCHEMA,
			netintSchema:	  NETINT_SCHEMA,
			filemountSchema:  FILEMOUNT_SCHEMA,
			nfsmountSchema:   NFSMOUNT_SCHEMA,
			serverSchema: 	  SERVER_SCHEMA,
			serviceSchema:	  SERVICE_SCHEMA,
			logSchema:		  LOG_SCHEMA,
			ctrlhisSchema:	  CTRLHIS_SCHEMA,
			ctrlschedSchema:  CTRLSCHED_SCHEMA,
			connspawnSchema:  CONNSPAWN_SCHEMA,
			metaSchema:  	  META_SCHEMA,
			olapSchema:  	  OLAP_SCHEMA,
			objspawnSchema:   OBJSPAWN_SCHEMA,
			storedprocSchema: STOREDPROC_SCHEMA,
			workspaceSchema:  WORKSPACE_SCHEMA,
			pooledworkspaceSchema: POOLEDWORKSPACE_SCHEMA,
			osservicesSchema: OSSERVICES_SCHEMA,
			activemqSchema:   ACTIVEMQ_SCHEMA,
			tomcatSchema:  	  TOMCAT_SCHEMA,
			tcruntimeSchema:  TCRUNTIME_SCHEMA,
			licenseSchema: 	  LICENSE_SCHEMA,
			datamartSchema:   DATAMART_SCHEMA,
			acmSchema:		  ACM_SCHEMA,
			apmSchema:		  APM_SCHEMA,	
			snapshotSchema:	  SNAPSHOT_SCHEMA,
			eventSchema:	  EVENT_SCHEMA,
			alertSchema:	  ALERT_SCHEMA,
			topSchema: 	  	  TOP_SCHEMA,
			dfSchema: 	  	  DF_SCHEMA,
			whoSchema: 	  	  WHO_SCHEMA,
		])
		
		log.info "Done loading table info. Invoking now."
    }
}
