#!/usr/bin/perl

# Perl script that gathers system information and outputs name=value pairs
# used for Hyperic measurement results

use warnings;
use strict;

my $data1 = <<END;
core file size          (blocks, -c) 0
data seg size           (kbytes, -d) unlimited
scheduling priority             (-e) 0
file size               (blocks, -f) unlimited
pending signals                 (-i) 127451
max locked memory       (kbytes, -l) 64
max memory size         (kbytes, -m) unlimited
open files                      (-n) 1024
pipe size            (512 bytes, -p) 8
POSIX message queues     (bytes, -q) 819200
real-time priority              (-r) 0
stack size              (kbytes, -s) 10240
cpu time               (seconds, -t) unlimited
max user processes              (-u) 1024
virtual memory          (kbytes, -v) unlimited
file locks                      (-x) unlimited
END

# Each element of @metrics array is a (name,value) list
my %metrics;

# Default to 1, set to 0 if we encounter an error
my $availability = 1;

# ==== Metric : Availability ====
$metrics{"Availability"} = $availability;

my $datauu = qx/ulimit -a 3>&-/;
#chomp(@uuu);
#print "@uuu";
#foreach (@uuu) {

foreach (split(/\n/,$datauu)) {
    if ($_ =~ m/core\sfile\ssize/) { $metrics{"corefilesize"} = get_ulimit_value($_); next;}
    if ($_ =~ m/data\sseg\ssize/)   { $metrics{"datasegsize"} = get_ulimit_value($_); next;}
    if ($_ =~ m/scheduling\spriority/) { $metrics{"schedulingpriority"} = get_ulimit_value($_); next;}
    if ($_ =~ m/file\ssize/) { $metrics{"filesize"} = get_ulimit_value($_); next;}
    if ($_ =~ m/pending\ssignals/) { $metrics{"pendingsignals"} = get_ulimit_value($_); next;}
    if ($_ =~ m/max\slocked\smemory/) { $metrics{"maxlockedmemory"} = get_ulimit_value($_); next;}
    if ($_ =~ m/max\smemory\ssize/) { $metrics{"maxmemorysize"} = get_ulimit_value($_); next;}
    if ($_ =~ m/open\sfiles/) { $metrics{"openfiles"} = get_ulimit_value($_); next;}
    if ($_ =~ m/pipe\ssize/) { $metrics{"pipesizes"} = get_ulimit_value($_); next;}
    if ($_ =~ m/POSIX\smessage\squeues/) { $metrics{"POSIXmessagequeues"} = get_ulimit_value($_); next;}
    if ($_ =~ m/real-time\spriority/) { $metrics{"realtimepriority"} = get_ulimit_value($_); next;}
    if ($_ =~ m/stack\ssize/) { $metrics{"stacksize"} = get_ulimit_value($_); next;}
    if ($_ =~ m/cpu\stime/) { $metrics{"cputime"} = get_ulimit_value($_); next;}
    if ($_ =~ m/max\suser\sprocesses/) { $metrics{"maxuserprocesses"} = get_ulimit_value($_); next;}
    if ($_ =~ m/virtual\smemory/) { $metrics{"virtualmemory"} = get_ulimit_value($_); next;}
    if ($_ =~ m/file\slocks/) { $metrics{"filelocks"} = get_ulimit_value($_); next;}
}
# Iterate over @metrics for output
my $key;
foreach $key (keys %metrics) { 
    print "$key=$metrics{$key}\n";
}

exit 0;

sub get_ulimit_value
{
	my $retval;
    $_ =~ /[a-z0-9-\s]+\(([^)]+)\)\s*(\w+)?$/;
    $retval=$2;
    $retval='-1' if ($2 =~ m/unlimited/); 
    return($retval);
}
